@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -243,7 +243,7 @@ param_put_cie_render1(gs_param_list * plist, gs_cie_render * pcrd,
                 /****** WRITE RenderTableTValues LIKE write_proc3 ******/
                 uint size = gx_cie_cache_size;
                 float *values =
-                    (float *)gs_alloc_byte_array(mem, size * m,
+                    (float *)gs_alloc_byte_array(mem, (size_t)size * m,
                                                  sizeof(float),
                                                  "write_proc3");
                 gs_param_float_array fa;
