@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -172,7 +172,7 @@ gx_install_Separation(gs_color_space * pcs, gs_gstate * pgs)
          */
         if (dev_proc(pgs->device, update_spot_equivalent_colors))
            code = dev_proc(pgs->device, update_spot_equivalent_colors)
-                                                        (pgs->device, pgs);
+                                                        (pgs->device, pgs, pcs);
     }
     return code;
 }
@@ -186,7 +186,7 @@ gx_set_overprint_Separation(const gs_color_space * pcs, gs_gstate * pgs)
     if (pcmap->use_alt_cspace)
         return gx_set_no_overprint(pgs);
     else {
-        gs_overprint_params_t   params;
+        gs_overprint_params_t params = { 0 };
 
         params.retain_any_comps = (((pgs->overprint && pgs->is_fill_color) ||
                                    (pgs->stroke_overprint && !pgs->is_fill_color)) &&
@@ -464,23 +464,39 @@ check_Separation_component_name(const gs_color_space * pcs, gs_gstate * pgs)
         pcolor_component_map->use_alt_cspace = false;
         return 0;
     }
-    /*
-     * Always use the alternate color space if the current device is
-     * using an additive color model.  Separations are only for use
-     * with a subtractive color model.  The exception is if we have a separation
-     * device and we are doing transparency blending in an additive color
-     * space.  In that case, the spots are kept separated and blended
-     * individually per the PDF specification.   Note however, if the spot is
-     * a CMYK process color and we are doing the blend in an additive color space
-     * the alternate color space is used.  This matches AR.
+    /* If our device is using an additive color model, then we need to
+     * consider using the alternative color space, as separations are
+     * generally only used with a subtractive color model. There are
+     * exceptions, however.
+     *
+     * If we don't support devn, then we will certainly have to use the
+     * alternative color space.
+     *
+     * If we are a pdf14 device, and we are doing transparency blending
+     * in an additive space, we need to keep the spots separated and
+     * blend them individually as per the PDF specification. Note
+     * however, if the spot is a CMYK process color and we are doing
+     * the blend in an additive color space the alternate color space
+     * is used.  This matches AR.
+     *
+     * Otherwise, we will always use the alternate colorspace, unless
+     * our device specifically claims to be a separation-supporting
+     * additive device. Possibly all additive devn devices should
+     * support this now, but I lack to confidence to make this change
+     * globally. Instead we'll just enable it on a device by device
+     * basis for now.
+     *
+     * This matches logic in check_DeviceN_component_names.
      */
-
-    if (!(dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0) &&
-        dev_proc(dev, dev_spec_op)(dev, gxdso_is_pdf14_device, NULL, 0)) &&
-        dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE) {
-        pcolor_component_map->use_alt_cspace = true;
-        return 0;
+    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE) {
+        if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0) == 0 ||
+            (dev_proc(dev, dev_spec_op)(dev, gxdso_is_pdf14_device, NULL, 0) == 0 &&
+             dev_proc(dev, dev_spec_op)(dev, gxdso_is_sep_supporting_additive_device, NULL, 0) == 0)) {
+            pcolor_component_map->use_alt_cspace = true;
+            return 0;
+        }
     }
+
     /*
      * Get the character string and length for the component name.
      */
