@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -120,11 +120,11 @@ gx_lookup_fm_pair(gs_font * pfont, const gs_matrix *char_tm,
 
             if (pair->font == 0) {
                 pair->font = pfont;
-                if_debug2m('k', pfont->memory, "[k]updating pair 0x%lx with font 0x%lx\n",
-                           (ulong) pair, (ulong) pfont);
+                if_debug2m('k', pfont->memory, "[k]updating pair "PRI_INTPTR" with font "PRI_INTPTR"\n",
+                           (intptr_t)pair, (intptr_t)pfont);
             } else {
-                if_debug2m('k', pfont->memory, "[k]found pair 0x%lx: font=0x%lx\n",
-                           (ulong) pair, (ulong) pair->font);
+                if_debug2m('k', pfont->memory, "[k]found pair "PRI_INTPTR": font="PRI_INTPTR"\n",
+                           (intptr_t)pair, (intptr_t)pair->font);
             }
             code = gx_touch_fm_pair(dir, pair);
             if (code < 0)
@@ -158,8 +158,8 @@ gx_lookup_cached_char(const gs_font * pfont, const cached_fm_pair * pair,
             cc->wmode == wmode && cc_depth(cc) == depth
             ) {
             if_debug4m('K', pfont->memory,
-                       "[K]found 0x%lx (depth=%d) for glyph=0x%lx, wmode=%d\n",
-                       (ulong) cc, cc_depth(cc), (ulong) glyph, wmode);
+                       "[K]found "PRI_INTPTR" (depth=%d) for glyph=0x%lx, wmode=%d\n",
+                       (intptr_t)cc, cc_depth(cc), (ulong)glyph, wmode);
             return cc;
         }
         chi++;
@@ -189,6 +189,10 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
     gx_xfont *xf;
     byte *bits;
 
+    /* This is only to silence a Coverity warning */
+    cdev.target = NULL;
+    cdev.cpath = NULL;
+
   top:code = gx_path_current_point_inline(pgs, &pt);
     if (code < 0)
         return code;
@@ -211,7 +215,8 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
                               "[K]bits");
         else
             dmputs(penum->memory, "[K]no bits\n");
-        dmlprintf3(penum->memory, "[K]copying 0x%lx, offset=(%g,%g)\n", (ulong) cc,
+        dmlprintf3(penum->memory, "[K]copying "PRI_INTPTR", offset=(%g,%g)\n",
+                   (intptr_t) cc,
                    fixed2float(-cc->offset.x),
                    fixed2float(-cc->offset.y));
         dmlprintf6(penum->memory, "   at (%g,%g)+(%d,%d)->(%d,%d)\n",
@@ -238,8 +243,11 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
         if_debug0m('K', penum->memory, "[K](clipping)\n");
     }
     code = gx_set_dev_color(pgs);
-    if (code != 0)
+    if (code != 0) {
+        if (imaging_dev == (gx_device *) & cdev)
+            gx_destroy_clip_device_on_stack(&cdev);
         return code;
+    }
     /* If an xfont can render this character, use it. */
     if (xg != gx_no_xglyph && (xf = cc_pair(cc)->xfont) != 0) {
         int cx = x + fixed2int(cc->offset.x);
@@ -256,12 +264,15 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
                                         imaging_dev, cx, cy,
                                         pdevc->colors.pure, 0);
             if_debug8m('K', penum->memory,
-                       "[K]render_char display: xfont=0x%lx, glyph=0x%lx\n\tdev=0x%lx(%s) x,y=%d,%d, color=0x%lx => %d\n",
-                       (ulong) xf, (ulong) xg, (ulong) imaging_dev,
+                       "[K]render_char display: xfont="PRI_INTPTR", glyph=0x%lx\n\tdev="PRI_INTPTR"(%s) x,y=%d,%d, color=0x%lx => %d\n",
+                       (intptr_t)xf, (ulong)xg, (intptr_t)imaging_dev,
                        imaging_dev->dname, cx, cy,
                        (ulong) pdevc->colors.pure, code);
-            if (code == 0)
+            if (code == 0) {
+                if (imaging_dev == (gx_device *) & cdev)
+                    gx_destroy_clip_device_on_stack(&cdev);
                 return_check_interrupt(penum->memory, 0);
+            }
         }
         /* Can't render directly.  If we don't have a bitmap yet, */
         /* get it from the xfont now. */
@@ -274,11 +285,14 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
                                        (gx_device *) & mdev, cx - x, cy - y,
                                                      (gx_color_index) 1, 1);
             if_debug7m('K', penum->memory,
-                       "[K]render_char to bits: xfont=0x%lx, glyph=0x%lx\n\tdev=0x%lx(%s) x,y=%d,%d => %d\n",
-                      (ulong) xf, (ulong) xg, (ulong) & mdev,
+                       "[K]render_char to bits: xfont="PRI_INTPTR", glyph=0x%lx\n\tdev="PRI_INTPTR"(%s) x,y=%d,%d => %d\n",
+                      (intptr_t)xf, (ulong) xg, (intptr_t)&mdev,
                       mdev.dname, cx - x, cy - y, code);
-            if (code != 0)
+            if (code != 0) {
+                if (imaging_dev == (gx_device *) & cdev)
+                    gx_destroy_clip_device_on_stack(&cdev);
                 return_check_interrupt(penum->memory, 1);
+            }
             gx_add_char_bits(cc_pair(cc)->font->dir,
                              cc, &scale_log2_1);
             /* gx_add_char_bits may change width, height, */
@@ -329,8 +343,11 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
                 return_check_interrupt(penum->memory, 0);
             /* copy_alpha failed, construct a monobit mask. */
             bits = compress_alpha_bits(cc, penum->memory->non_gc_memory);
-            if (bits == 0)
+            if (bits == 0) {
+                if (imaging_dev == (gx_device *) & cdev)
+                    gx_destroy_clip_device_on_stack(&cdev);
                 return 1;	/* VMerror, but recoverable */
+            }
         }
         code = (*dev_proc(imaging_dev, copy_mono))
             (imaging_dev, bits, 0, bitmap_raster(w), gs_no_id,
@@ -340,9 +357,11 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
     if (depth > 1) {		/* Complex color or fill_mask / copy_alpha failed, */
         /* construct a monobit mask. */
         bits = compress_alpha_bits(cc, penum->memory->non_gc_memory);
-        if (bits == 0)
+        if (bits == 0) {
+            if (imaging_dev == (gx_device *) & cdev)
+                gx_destroy_clip_device_on_stack(&cdev);
             return 1;		/* VMerror, but recoverable */
-
+        }
     } {				/* Use imagemask to render the character. */
         gs_memory_t *mem = penum->memory->non_gc_memory;
         gs_image_enum *pie =
@@ -357,6 +376,8 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
             if (bits != cc_bits(cc))
                 gs_free_object(mem, bits,
                                "compress_alpha_bits");
+            if (imaging_dev == (gx_device *) & cdev)
+                gx_destroy_clip_device_on_stack(&cdev);
             return 1;		/* VMerror, but recoverable */
         }
         /* Make a matrix that will place the image */
@@ -386,6 +407,8 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
         gs_free_object(penum->memory->non_gc_memory, bits, "compress_alpha_bits");
     if (code > 0)
         code = 0;
+    if (imaging_dev == (gx_device *) & cdev)
+        gx_destroy_clip_device_on_stack(&cdev);
     return_check_interrupt(penum->memory, code);
 }
 
@@ -410,7 +433,7 @@ compress_alpha_bits(const cached_char * cc, gs_memory_t * mem)
     uint sskip = sraster - ((width * depth + 7) >> 3);
     uint draster = bitmap_raster(width);
     uint dskip = draster - ((width + 7) >> 3);
-    byte *mask = gs_alloc_bytes(mem, draster * height,
+    byte *mask = gs_alloc_bytes(mem, (size_t)draster * height,
                                 "compress_alpha_bits");
     const byte *sptr = data;
     byte *dptr = mask;
