@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -100,7 +100,7 @@ make_mid_default(gx_device **pmidev, gx_device *dev, int width, int height,
                  gs_memory_t *mem)
 {
     gx_device_memory *midev =
-        gs_alloc_struct(mem, gx_device_memory, &st_device_memory,
+        gs_alloc_struct_immovable(mem, gx_device_memory, &st_device_memory,
                         "make_mid_default");
     int code;
 
@@ -136,20 +136,31 @@ make_mcde_default(gx_device *dev, const gs_gstate *pgs,
                   const gs_int_point *origin)
 {
     gx_device_memory *const mdev = (gx_device_memory *)midev;
-    gx_device_mask_clip *mcdev =
-        gs_alloc_struct(mem, gx_device_mask_clip, &st_device_mask_clip,
-                        "make_mcde_default");
+    gx_device_mask_clip *mcdev = NULL;
     gx_strip_bitmap bits;	/* only gx_bitmap */
     int code;
 
+    /* The gx_strip_bitmap structure defines (via gs_tile_bitmap_common)
+     * rep_width and rep_height as being of type 'ushort', device width and
+     * height are of type 'int'. Make sure we don't overflow because that
+     * will lead to memory corruption.
+     */
+    if (mdev->width > ARCH_MAX_USHORT || mdev->height > ARCH_MAX_USHORT)
+        return_error(gs_error_rangecheck);
+
+    mcdev = gs_alloc_struct(mem, gx_device_mask_clip, &st_device_mask_clip,
+                        "make_mcde_default");
+
     if (mcdev == 0)
         return_error(gs_error_VMerror);
     bits.data = mdev->base;
     bits.raster = mdev->raster;
-    bits.size.x = mdev->width;
-    bits.size.y = mdev->height;
+
+    bits.size.x = bits.rep_width = mdev->width;
+    bits.size.y = bits.rep_height = mdev->height;
     bits.id = gx_no_bitmap_id;
     bits.num_planes = 1;
+    bits.rep_shift = bits.shift = 0;
     code = gx_mask_clip_initialize(mcdev, &gs_mask_clip_device,
                                    (const gx_bitmap *)&bits, dev,
                                    origin->x, origin->y, mem);
@@ -208,7 +219,8 @@ gx_begin_image3_generic(gx_device * dev,
     int code;
 
     /* Validate the parameters. */
-    if (pim->Height <= 0 || pim->MaskDict.Height <= 0)
+    if (pim->Width <= 0 || pim->MaskDict.Width <= 0 ||
+        pim->Height <= 0 || pim->MaskDict.Height <= 0)
         return_error(gs_error_rangecheck);
     switch (pim->InterleaveType) {
         default:
@@ -333,7 +345,7 @@ gx_begin_image3_generic(gx_device * dev,
         /* Allocate row buffers for the mask and pixel data. */
         penum->pixel_data =
             gs_alloc_bytes(mem,
-                           (penum->pixel_width * pim->BitsPerComponent *
+                           ((size_t)penum->pixel_width * pim->BitsPerComponent *
                             penum->num_components + 7) >> 3,
                            "gx_begin_image3(pixel_data)");
         penum->mask_data =
