@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -111,10 +111,10 @@ gx_path_init_contents(gx_path * ppath)
     ppath->bbox_set = 0;
     ppath->bbox_accurate = 0;
     ppath->last_charpath_segment = 0;
-    ppath->bbox.p.x = max_int;
-    ppath->bbox.p.y = max_int;
-    ppath->bbox.q.x = min_int;
-    ppath->bbox.q.y = min_int;
+    ppath->bbox.p.x = max_fixed;
+    ppath->bbox.p.y = max_fixed;
+    ppath->bbox.q.x = min_fixed;
+    ppath->bbox.q.y = min_fixed;
 }
 
 /*
@@ -137,8 +137,10 @@ gx_path_init_contained_shared(gx_path * ppath, const gx_path * shared,
 {
     if (shared) {
         if (shared->segments == &shared->local_segments) {
+#ifdef DEBUG
             lprintf1("Attempt to share (local) segments of path "PRI_INTPTR"!\n",
                      (intptr_t)shared);
+#endif
             return_error(gs_error_Fatal);
         }
         *ppath = *shared;
@@ -172,8 +174,10 @@ gx_path_alloc_shared(const gx_path * shared, gs_memory_t * mem,
     ppath->procs = &default_path_procs;
     if (shared) {
         if (shared->segments == &shared->local_segments) {
+#ifdef DEBUG
             lprintf1("Attempt to share (local) segments of path "PRI_INTPTR"!\n",
                      (intptr_t)shared);
+#endif
             gs_free_object(mem, ppath, cname);
             return 0;
         }
@@ -203,8 +207,10 @@ gx_path_init_local_shared(gx_path * ppath, const gx_path * shared,
 {
     if (shared) {
         if (shared->segments == &shared->local_segments) {
+#ifdef DEBUG
             lprintf1("Attempt to share (local) segments of path "PRI_INTPTR"!\n",
                      (intptr_t)shared);
+#endif
             return_error(gs_error_Fatal);
         }
         *ppath = *shared;
@@ -221,6 +227,28 @@ gx_path_init_local_shared(gx_path * ppath, const gx_path * shared,
     return 0;
 }
 
+void gx_path_preinit_local_rectangle(gx_path *ppath, gs_memory_t *mem)
+{
+    rc_init_free(&ppath->local_segments, mem, 1, NULL);
+    ppath->segments = &ppath->local_segments;
+    ppath->box_last = 0;
+    ppath->first_subpath = ppath->current_subpath = 0;
+    ppath->subpath_count = 0;
+    ppath->curve_count = 0;
+    path_update_newpath(ppath);
+    ppath->bbox_set = 1;
+    ppath->bbox_accurate = 1;
+    ppath->last_charpath_segment = 0;
+    ppath->memory = mem;
+    ppath->allocation = path_allocated_on_stack;
+    ppath->procs = &default_path_procs;
+}
+
+void gx_path_init_local_rectangle(gx_path *ppath, gs_fixed_rect *rect)
+{
+    ppath->bbox = *rect;
+}
+
 /*
  * Initialize a stack-allocated pseudo-path for computing a bbox
  * for a dynamic path.
@@ -243,6 +271,16 @@ gx_path_init_bbox_accumulator(gx_path * ppath)
     ppath->procs = &path_bbox_procs;
 }
 
+/* Guarantee that a path's segments are not shared with any other path. */
+static inline int path_unshare(gx_path *ppath)
+{
+    int code = 0;
+
+    if (gx_path_is_shared(ppath))
+        code = path_alloc_copy(ppath);
+    return code;
+}
+
 /*
  * Ensure that a path owns its segments, by copying the segments if
  * they currently have multiple references.
@@ -250,11 +288,7 @@ gx_path_init_bbox_accumulator(gx_path * ppath)
 int
 gx_path_unshare(gx_path * ppath)
 {
-    int code = 0;
-
-    if (gx_path_is_shared(ppath))
-        code = path_alloc_copy(ppath);
-    return code;
+    return path_unshare(ppath);
 }
 
 /*
@@ -390,45 +424,51 @@ rc_free_path_segments(gs_memory_t * mem, void *vpsegs, client_name_t cname)
 
 /* ------ Incremental path building ------ */
 
-/* Guarantee that a path's segments are not shared with any other path. */
-#define path_unshare(ppath)\
-  BEGIN\
-    if ( gx_path_is_shared(ppath) ) {\
-      int code_;\
-      if( (code_ = path_alloc_copy(ppath)) < 0 ) return code_;\
-    }\
-  END
-
 /* Macro for opening the current subpath. */
 /* ppath points to the path; sets psub to ppath->current_subpath. */
-#define path_open()\
-  BEGIN\
-    if ( !path_is_drawing(ppath) ) {\
-      int code_;\
-      if ( !path_position_valid(ppath) )\
-        return_error(gs_error_nocurrentpoint);\
-      code_ = gx_path_new_subpath(ppath);\
-      if ( code_ < 0 ) return code_;\
-    }\
-  END
+static inline int path_open(gx_path *ppath)
+{
+    int code = 0;
+    if ( !path_is_drawing(ppath) ) {
+      if ( path_position_valid(ppath) ) {
+          code = gx_path_new_subpath(ppath);
+      }
+      else {
+          code = gs_note_error(gs_error_nocurrentpoint);
+      }
+    }
+    return code;
+}
 
 /* Macros for allocating path segments. */
 /* Note that they assume that ppath points to the path. */
 /* We have to split the macro into two because of limitations */
 /* on the size of a single statement (sigh). */
-#define path_alloc_segment(pseg,ctype,pstype,stype,snotes,cname)\
-  path_unshare(ppath);\
-  psub = ppath->current_subpath;\
-  if( !(pseg = gs_alloc_struct(gs_memory_stable(ppath->memory), ctype,\
-                               pstype, cname)) )\
-    return_error(gs_error_VMerror);\
-  pseg->type = stype, pseg->notes = snotes, pseg->next = 0
-#define path_alloc_link(pseg)\
-  { segment *prev = psub->last;\
-    prev->next = (segment *)pseg;\
-    pseg->prev = prev;\
-    psub->last = (segment *)pseg;\
-  }
+static inline int path_alloc_segment(gx_path *ppath, segment **ppseg, subpath **ppsub, gs_memory_type_ptr_t pstype, segment_type seg_type, ushort notes, client_name_t cname)
+{
+    int code;
+
+    code = path_unshare(ppath);
+    if (code < 0) return code;
+    if (ppsub)
+        *ppsub = ppath->current_subpath;
+
+    if( !(*ppseg = gs_alloc_struct(gs_memory_stable(ppath->memory), segment, pstype, cname)) )
+      return_error(gs_error_VMerror);
+    (*ppseg)->type = seg_type;
+    (*ppseg)->notes = notes;
+    (*ppseg)->next = 0;
+
+    return 0;
+}
+
+static inline void path_alloc_link(segment *pseg, subpath *psub)
+{
+    segment *prev = psub->last;
+    prev->next = (segment *)pseg;
+    pseg->prev = prev;
+    psub->last = (segment *)pseg;
+}
 
 /* Make a new path (newpath). */
 int
@@ -440,9 +480,12 @@ gx_path_new(gx_path * ppath)
         int code = path_alloc_segments(&ppath->segments, ppath->memory,
                                        "gx_path_new");
 
-        rc_decrement(psegs, "gx_path_new");
-        if (code < 0)
+        if (code < 0) {
+            /* Leave the path in a valid state, despite the error */
+            ppath->segments = psegs;
             return code;
+        }
+        rc_decrement(psegs, "gx_path_new");
     } else {
         rc_free_path_segments_local(psegs->rc.memory, psegs, "gx_path_new");
     }
@@ -457,9 +500,12 @@ gx_path_new_subpath(gx_path * ppath)
 {
     subpath *psub;
     subpath *spp;
+    int code;
+
+    code = path_alloc_segment(ppath, (segment **)&spp, &psub, &st_subpath, s_start, sn_none, "gx_path_new_subpath");
+    if (code < 0)
+        return code;
 
-    path_alloc_segment(spp, subpath, &st_subpath, s_start, sn_none,
-                       "gx_path_new_subpath");
     spp->last = (segment *) spp;
     spp->curve_count = 0;
     spp->is_closed = 0;
@@ -571,13 +617,15 @@ gz_path_add_line_notes(gx_path * ppath, fixed x, fixed y, segment_notes notes)
 {
     subpath *psub;
     line_segment *lp;
+    int code;
 
     if (ppath->bbox_set)
         check_in_bbox(ppath, x, y);
-    path_open();
-    path_alloc_segment(lp, line_segment, &st_line, s_line, notes,
-                       "gx_path_add_line");
-    path_alloc_link(lp);
+    code = path_open(ppath);
+    if (code < 0) return code;
+    code = path_alloc_segment(ppath, (segment **)&lp, &psub, &st_line, s_line, notes, "gx_path_add_line");
+    if (code < 0) return code;
+    path_alloc_link((segment *)lp, psub);
     path_set_point(lp, x, y);
     path_update_draw(ppath);
     trace_segment("[P]", ppath->memory, (segment *) lp);
@@ -601,13 +649,15 @@ gz_path_add_gap_notes(gx_path * ppath, fixed x, fixed y, segment_notes notes)
 {
     subpath *psub;
     line_segment *lp;
+    int code;
 
     if (ppath->bbox_set)
         check_in_bbox(ppath, x, y);
-    path_open();
-    path_alloc_segment(lp, line_segment, &st_line, s_gap, notes,
-                       "gx_path_add_gap");
-    path_alloc_link(lp);
+    code = path_open(ppath);
+    if (code < 0) return code;
+    code = path_alloc_segment(ppath, (segment **)&lp, &psub, &st_line, s_gap, notes, "gx_path_add_gap");
+    if (code < 0) return code;
+    path_alloc_link((segment *)lp, psub);
     path_set_point(lp, x, y);
     path_update_draw(ppath);
     trace_segment("[P]", ppath->memory, (segment *) lp);
@@ -635,8 +685,12 @@ gx_path_add_lines_notes(gx_path *ppath, const gs_fixed_point *ppts, int count,
 
     if (count <= 0)
         return 0;
-    path_unshare(ppath);
-    path_open();
+    code = path_unshare(ppath);
+    if (code < 0)
+        return code;
+
+    code = path_open(ppath);
+    if (code < 0) return code;
     psub = ppath->current_subpath;
     prev = psub->last;
     /*
@@ -687,13 +741,15 @@ gx_path_add_dash_notes(gx_path * ppath, fixed x, fixed y, fixed dx, fixed dy, se
 {
     subpath *psub;
     dash_segment *lp;
+    int code;
 
     if (ppath->bbox_set)
         check_in_bbox(ppath, x, y);
-    path_open();
-    path_alloc_segment(lp, dash_segment, &st_dash, s_dash, notes,
-                       "gx_dash_add_dash");
-    path_alloc_link(lp);
+    code = path_open(ppath);
+    if (code < 0) return code;
+    code = path_alloc_segment(ppath, (segment **)&lp, &psub, &st_dash, s_dash, notes, "gx_dash_add_dash");
+    if (code < 0) return code;
+    path_alloc_link((segment *)lp, psub);
     path_set_point(lp, x, y);
     lp->tangent.x = dx;
     lp->tangent.y = dy;
@@ -737,16 +793,18 @@ gz_path_add_curve_notes(gx_path * ppath,
 {
     subpath *psub;
     curve_segment *lp;
+    int code;
 
     if (ppath->bbox_set) {
         check_in_bbox(ppath, x1, y1);
         check_in_bbox(ppath, x2, y2);
         check_in_bbox(ppath, x3, y3);
     }
-    path_open();
-    path_alloc_segment(lp, curve_segment, &st_curve, s_curve, notes,
-                       "gx_path_add_curve");
-    path_alloc_link(lp);
+    code = path_open(ppath);
+    if (code < 0) return code;
+    code = path_alloc_segment(ppath, (segment **)&lp, &psub, &st_curve, s_curve, notes, "gx_path_add_curve");
+    if (code < 0) return code;
+    path_alloc_link((segment *)lp, psub);
     lp->p1.x = x1;
     lp->p1.y = y1;
     lp->p2.x = x2;
@@ -809,8 +867,12 @@ fixed x3, fixed y3, fixed xt, fixed yt, double fraction, segment_notes notes)
 int
 gx_path_add_path(gx_path * ppath, gx_path * ppfrom)
 {
-    path_unshare(ppfrom);
-    path_unshare(ppath);
+    int code = path_unshare(ppfrom);
+    if (code < 0)
+        return code;
+    code = path_unshare(ppath);
+    if (code < 0)
+        return code;
     if (ppfrom->first_subpath) {	/* i.e. ppfrom not empty */
         if (ppath->first_subpath) {	/* i.e. ppath not empty */
             subpath *psub = ppath->current_subpath;
@@ -897,9 +959,9 @@ gz_path_close_subpath_notes(gx_path * ppath, segment_notes notes)
         if (code < 0)
             return code;
     }
-    path_alloc_segment(lp, line_close_segment, &st_line_close,
-                       s_line_close, notes, "gx_path_close_subpath");
-    path_alloc_link(lp);
+    code = path_alloc_segment(ppath, (segment **)&lp, &psub, &st_line_close, s_line_close, notes, "gx_path_close_subpath");
+    if (code < 0) return code;
+    path_alloc_link((segment *)lp, psub);
     path_set_point(lp, psub->pt.x, psub->pt.y);
     lp->sub = psub;
     psub->is_closed = 1;
@@ -1026,7 +1088,7 @@ gx_print_segment(const gs_memory_t *mem, const segment * pseg)
     double py = fixed2float(pseg->pt.y);
     char out[80];
 
-    gs_sprintf(out, PRI_INTPTR "<"PRI_INTPTR","PRI_INTPTR">:%u",
+    gs_snprintf(out, sizeof(out), PRI_INTPTR "<"PRI_INTPTR","PRI_INTPTR">:%u",
                (intptr_t)pseg, (intptr_t)pseg->prev, (intptr_t)pseg->next, pseg->notes);
     switch (pseg->type) {
         case s_start:{
