@@ -211,16 +211,26 @@ static media_t media_codes[] = {
 /* ------------------------- 1 bit Monochrome ---------------------------- */
 /***************************************************************************/
 
-static const gx_device_procs bjcmono_procs =
-prn_color_params_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close,
-                 NULL, NULL,
-                 gdev_bjc_get_params, gdev_bjc_put_params);
+static void
+bjcmono_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs(dev);
+
+    set_dev_proc(dev, get_params, gdev_bjc_get_params);
+    set_dev_proc(dev, put_params, gdev_bjc_put_params);
+    set_dev_proc(dev, map_color_rgb, gx_default_w_b_map_color_rgb);
+    set_dev_proc(dev, map_rgb_color, gx_default_gray_fast_encode);
+    set_dev_proc(dev, encode_color, gx_default_gray_fast_encode);
+    set_dev_proc(dev, decode_color, gx_default_w_b_map_color_rgb);
+}
+
 
 const gx_device_bjc_printer gs_bjcmono_device =
-bjc_device(bjcmono_procs, "bjcmono",
+bjc_device(bjcmono_initialize_device_procs, "bjcmono",
            DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
            X_DPI, Y_DPI,
-           (3.4 / 25.4), (7.0 / 25.4), (3.4 / 25.4), (3.0 / 25.4),
+           (float)(3.4 / 25.4), (float)(7.0 / 25.4),
+           (float)(3.4 / 25.4), (float)(3.0 / 25.4),
            1,                   /* num components */
            1,                   /* depth  */
            1,                   /* max gray */
@@ -233,16 +243,27 @@ bjc_device(bjcmono_procs, "bjcmono",
 /* -------------------------- 8 bit Grayscale ---------------------------- */
 /***************************************************************************/
 
-static const gx_device_procs bjcgray_procs =
-prn_color_params_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close,
-                 gx_default_gray_map_rgb_color, gx_default_gray_map_color_rgb,
-                 gdev_bjc_get_params, gdev_bjc_put_params);
+static void
+bjcgray_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_gray(dev);
+
+    set_dev_proc(dev, get_params, gdev_bjc_get_params);
+    set_dev_proc(dev, put_params, gdev_bjc_put_params);
+
+    /* The prn macros used in previous versions of the code leave
+     * encode_color and decode_color set to NULL (which are then rewritten
+     * by the system to the default. For compatibility we do the same. */
+    set_dev_proc(dev, encode_color, NULL);
+    set_dev_proc(dev, decode_color, NULL);
+}
 
 const gx_device_bjc_printer gs_bjcgray_device =
-bjc_device(bjcgray_procs, "bjcgray",
+bjc_device(bjcgray_initialize_device_procs, "bjcgray",
            DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
            X_DPI, Y_DPI,
-           (3.4 / 25.4), (7.0 / 25.4), (3.4 / 25.4), (3.0 / 25.4),
+           (float)(3.4 / 25.4), (float)(7.0 / 25.4),
+           (float)(3.4 / 25.4), (float)(3.0 / 25.4),
            1,                   /* num components */
            8,                   /* depth  */
            255,                 /* max gray */
@@ -255,16 +276,27 @@ bjc_device(bjcgray_procs, "bjcgray",
 /* --------------------------- 3 bit CMYK Color -------------------------- */
 /***************************************************************************/
 
-static const gx_device_procs bjc_cmykcolor_procs =
-bjc_cmyk_param_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close,
-                     cmyk_1bit_map_color_rgb, cmyk_1bit_map_cmyk_color,
-                     gdev_bjc_get_params, gdev_bjc_put_params);
+static void
+bjccmyk_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_cmyk1(dev);
+
+    set_dev_proc(dev, get_params, gdev_bjc_get_params);
+    set_dev_proc(dev, put_params, gdev_bjc_put_params);
+
+    /* The prn macros used in previous versions of the code leave
+     * encode_color and decode_color set to NULL (which are then rewritten
+     * by the system to the default. For compatibility we do the same. */
+    set_dev_proc(dev, encode_color, NULL);
+    set_dev_proc(dev, decode_color, NULL);
+}
 
 const gx_device_bjc_printer gs_bjccmyk_device =
-bjc_device(bjc_cmykcolor_procs, "bjccmyk",
+bjc_device(bjccmyk_initialize_device_procs, "bjccmyk",
            DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
            X_DPI, Y_DPI,
-           (3.4 / 25.4), (7.0 / 25.4), (3.4 / 25.4), (3.0 / 25.4),
+           (float)(3.4 / 25.4), (float)(7.0 / 25.4),
+           (float)(3.4 / 25.4), (float)(3.0 / 25.4),
            4,                   /* num components */
            4,                   /* depth  */
            1,                   /* max gray */
@@ -277,16 +309,27 @@ bjc_device(bjc_cmykcolor_procs, "bjccmyk",
 /* --------------------------- 24 bit TrueColor -------------------------- */
 /***************************************************************************/
 
-static const gx_device_procs bjc_truecolor_procs =
-bjc_cmyk_param_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close,
-                     cmyk_8bit_map_color_rgb, cmyk_8bit_map_cmyk_color,
-                     gdev_bjc_get_params, gdev_bjc_put_params);
+static void
+bjc_truecolor_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_cmyk8(dev);
+
+    set_dev_proc(dev, get_params, gdev_bjc_get_params);
+    set_dev_proc(dev, put_params, gdev_bjc_put_params);
+
+    /* The prn macros used in previous versions of the code leave
+     * encode_color and decode_color set to NULL (which are then rewritten
+     * by the system to the default. For compatibility we do the same. */
+    set_dev_proc(dev, encode_color, NULL);
+    set_dev_proc(dev, decode_color, NULL);
+}
 
 const gx_device_bjc_printer gs_bjccolor_device =
-bjc_device(bjc_truecolor_procs, "bjccolor",
+bjc_device(bjc_truecolor_initialize_device_procs, "bjccolor",
            DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
            X_DPI, Y_DPI,
-           (3.4 / 25.4), (7.0 / 25.4), (3.4 / 25.4), (3.0 / 25.4),
+           (float)(3.4 / 25.4), (float)(7.0 / 25.4),
+           (float)(3.4 / 25.4), (float)(3.0 / 25.4),
            4,                   /* num components */
            32,                  /* depth  */
            255,                 /* max gray */
@@ -301,7 +344,7 @@ bjc_device(bjc_truecolor_procs, "bjccolor",
 
 /*
 static int
-bjc_print_page(gx_device_printer * pdev, FILE * file)
+bjc_print_page(gx_device_printer * pdev, gp_file * file)
 {
 #define ppdev ((gx_device_bjc_printer *) pdev)
 #define prdev ((gx_device_printer *) pdev)
@@ -328,7 +371,7 @@ bjc_print_page(gx_device_printer * pdev, FILE * file)
 done:
     gs_free_object(pdev->memory, row, "bjc file buffer");
 
-    fwrite((const char *) alma, 512, 1, file);
+    gp_fwrite((const char *) alma, 512, 1, file);
     return code;
 
 #undef ppdev
@@ -336,15 +379,15 @@ done:
 */
 
 void
-bjc_put_bjl_command(FILE * file, int bjl_command)
+bjc_put_bjl_command(gp_file * file, int bjl_command)
 {
     BJL_command *command = BJL_command_set;
     for( ; command->string; command++)
         if(command->numeric == bjl_command) break;
     if(command->string) {
-      fwrite((const char *)"\033[K\002\000\000\037BJLSTART\012", 16, 1, file);
-      fwrite(command->string, command->length, 1, file);
-      fwrite((const char *)"\012BJLEND\012", 8, 1, file); }
+      gp_fwrite((const char *)"\033[K\002\000\000\037BJLSTART\012", 16, 1, file);
+      gp_fwrite(command->string, command->length, 1, file);
+      gp_fwrite((const char *)"\012BJLEND\012", 8, 1, file); }
 }
 
 /* ------ Get/put parameters ------ */
@@ -386,8 +429,9 @@ gdev_bjc_get_params(gx_device * pdev, gs_param_list * plist)
 {
     const gx_device_bjc_printer * ppdev = (gx_device_bjc_printer *)pdev;
 
-    int code = gdev_prn_get_params(pdev, plist);
-    if (code < 0 ||
+    int code;
+    (void) (
+        (code = gdev_prn_get_params(pdev, plist)) < 0 ||
         (code = param_write_string(plist, "PrinterType",
                  &paramValueToParam(strPrinterType, ppdev->printerType)->p_string)) < 0 ||
         (code = param_write_string(plist, "Feeder",
@@ -413,8 +457,8 @@ gdev_bjc_get_params(gx_device * pdev, gs_param_list * plist)
         (code = param_write_float(plist, "Gamma", &ppdev->gamma)) < 0 ||
         (code = param_write_float(plist, "RedGamma", &ppdev->redGamma)) < 0 ||
         (code = param_write_float(plist, "GreenGamma", &ppdev->greenGamma)) < 0 ||
-        (code = param_write_float(plist, "BlueGamma", &ppdev->blueGamma)) < 0)
-        return code;
+        (code = param_write_float(plist, "BlueGamma", &ppdev->blueGamma)) < 0
+        );
     return code;
 
 }
@@ -553,15 +597,16 @@ label:							\
 }
 
 static int
-bjc_print_page_mono(gx_device_printer * pdev, FILE * file)
+bjc_print_page_mono(gx_device_printer * pdev, gp_file * file)
 {
 #define ppdev ((gx_device_bjc_printer *) pdev)
 #define prdev ((gx_device_printer *) pdev)
 
     uint raster = gdev_prn_raster(pdev);
     uint cmplen;
+    size_t rsize = (raster << 1) + 1;
     byte *row = gs_alloc_bytes(pdev->memory, raster, "bjc mono file buffer");
-    byte *cmp = gs_alloc_bytes(pdev->memory, (raster << 1) + 1,
+    byte *cmp = gs_alloc_bytes(pdev->memory, rsize,
                                "bjc mono comp buffer"); /*worst case */
     byte *outrow; /* misc variable for send a row */
     int y;
@@ -570,8 +615,8 @@ bjc_print_page_mono(gx_device_printer * pdev, FILE * file)
                   ((ppdev->ink & INK_K) ? 0x11: 0x10)); /* black or color */
     char ink   = 0x01; /* regular ink type */
     char compress = (ppdev->compress == true ? 0x01 : 0x00); /* compression or not */
-    int x_resolution = pdev->HWResolution[0];
-    int y_resolution = pdev->HWResolution[1];
+    int x_resolution = (int)pdev->HWResolution[0];
+    int y_resolution = (int)pdev->HWResolution[1];
     int length = 0/*x71*/, lm = 0/*x01*/, rm = 0/*x01*/, top = 0/*x50*/;
     byte inkc = ppdev->ink;
     byte mask_array[] = { 0xff, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe };
@@ -624,17 +669,18 @@ bjc_print_page_mono(gx_device_printer * pdev, FILE * file)
 }
 
 static int
-bjc_print_page_gray(gx_device_printer * pdev, FILE * file)
+bjc_print_page_gray(gx_device_printer * pdev, gp_file * file)
 {
 #define ppdev ((gx_device_bjc_printer *) pdev)
 #define prdev ((gx_device_printer *) pdev)
 
     uint width =   pdev->width; /* Because grayscale */
     uint raster = (pdev->width >> 3) + ( (pdev->width % 8) ? 1:0);
+    size_t rsize = (raster << 1) + 1;
     uint cmplen;
     byte *row = gs_alloc_bytes(pdev->memory, width, "bjc gray file buffer");
     byte *dit = gs_alloc_bytes(pdev->memory, raster, "bjc gray dither buffer");
-    byte *cmp = gs_alloc_bytes(pdev->memory, (raster << 1) + 1,
+    byte *cmp = gs_alloc_bytes(pdev->memory, rsize,
                                "bjc gray comp buffer"); /*worst case */
     byte *out; /* misc variable for send a row */
     int y;
@@ -643,8 +689,8 @@ bjc_print_page_gray(gx_device_printer * pdev, FILE * file)
                   ((ppdev->ink & INK_K) ? 0x11: 0x10)); /* black or color */
     char ink   = 0x01; /* regular ink type */
     char compress = (ppdev->compress == true ? 0x01 : 0x00); /* compression or not */
-    int x_resolution = pdev->HWResolution[0];
-    int y_resolution = pdev->HWResolution[1];
+    int x_resolution = (int)pdev->HWResolution[0];
+    int y_resolution = (int)pdev->HWResolution[1];
     int length = 0/*x71*/, lm = 0/*x01*/, rm = 0/*x01*/, top = 0/*x50*/;
     byte inkc = ppdev->ink;
     byte mask_array[] = { 0xff, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe };
@@ -711,7 +757,7 @@ bjc_print_page_gray(gx_device_printer * pdev, FILE * file)
 }
 
 static int
-bjc_print_page_cmyk(gx_device_printer * pdev, FILE * file)
+bjc_print_page_cmyk(gx_device_printer * pdev, gp_file * file)
 {
 #define ppdev ((gx_device_bjc_printer *) pdev)
 #define prdev ((gx_device_printer *) pdev)
@@ -719,9 +765,10 @@ bjc_print_page_cmyk(gx_device_printer * pdev, FILE * file)
     uint raster = bitmap_raster(pdev->width);
     uint a_raster;                             /* a tmp variable */
     uint cmplen;
-    byte *row = gs_alloc_bytes(pdev->memory, raster*4,
+    size_t cmpsize = (raster << 1) + 1, rsize = raster * 4;
+    byte *row = gs_alloc_bytes(pdev->memory, rsize,
                                "bjc cmyk file buffer"); /* one for each component */
-    byte *cmp = gs_alloc_bytes(pdev->memory, (raster << 1) + 1,
+    byte *cmp = gs_alloc_bytes(pdev->memory, cmpsize,
                                "bjc cmyk comp buffer"); /*worst case */
     byte *rows[4];
     byte *outrow; /* misc variable for send a row */
@@ -730,10 +777,10 @@ bjc_print_page_cmyk(gx_device_printer * pdev, FILE * file)
     char color = 0x10; /* color */
     char ink   = 0x01; /* regular ink type */
     char compress = (ppdev->compress == true ? 0x01 : 0x00); /* compression or not */
-    int x_resolution = pdev->HWResolution[0];
-    int y_resolution = pdev->HWResolution[1];
+    int x_resolution = (int)pdev->HWResolution[0];
+    int y_resolution = (int)pdev->HWResolution[1];
     int length = 0/*x71*/, lm = 0/*x01*/, rm = 0/*x01*/, top = 0/*x50*/;
-    int plane;
+    int plane, code = 0;
     byte mask_array[] = { 0xff, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe };
     byte lastmask = mask_array[pdev->width % 8];
     skip_t skipc;
@@ -760,9 +807,13 @@ bjc_print_page_cmyk(gx_device_printer * pdev, FILE * file)
     for (y = 0; y < pdev->height ; y++) {
 
         for (plane = 0; plane < 4; plane++) {   /* print each color component */
-            gx_render_plane_init(&render_plane, (gx_device *)pdev, plane);
-            gdev_prn_get_lines(pdev, y, 1, row + raster*plane, raster,
+            code = gx_render_plane_init(&render_plane, (gx_device *)pdev, plane);
+            if (code < 0)
+                goto error;
+            code = gdev_prn_get_lines(pdev, y, 1, row + raster*plane, raster,
                                &rows[plane], &a_raster, &render_plane);
+            if (code < 0)
+                goto error;
         }
 
         {
@@ -818,16 +869,17 @@ bjc_print_page_cmyk(gx_device_printer * pdev, FILE * file)
     bjc_put_FF(file);            /* eject a page */
     bjc_put_initialize (file);
 
+error:
     gs_free_object(pdev->memory, cmp, "bjc cmyk comp buffer");
     gs_free_object(pdev->memory, row, "bjc cmyk file buffer");
 
-    return 0;
+    return code;
 
 #undef ppdev
 }
 
 static int
-bjc_print_page_color(gx_device_printer * pdev, FILE * file)
+bjc_print_page_color(gx_device_printer * pdev, gp_file * file)
 {
 #define ppdev ((gx_device_bjc_printer *) pdev)
 #define prdev ((gx_device_printer *) pdev)
@@ -835,11 +887,12 @@ bjc_print_page_color(gx_device_printer * pdev, FILE * file)
     uint width =   pdev->width; /* Because grayscale */
     uint raster = (pdev->width >> 3) + ( (pdev->width % 8) ? 1:0);
     uint cmplen;
-    byte *row = gs_alloc_bytes(pdev->memory, width*4,
+    size_t cmpsize = (raster << 1) + 1, rsize = raster * 4, wsize = width * 4;
+    byte *row = gs_alloc_bytes(pdev->memory, wsize,
                                "bjc true file buffer"); /* one for each component */
-    byte *dit = gs_alloc_bytes(pdev->memory, raster*4,
+    byte *dit = gs_alloc_bytes(pdev->memory, rsize,
                                "bjc true dither buffer");
-    byte *cmp = gs_alloc_bytes(pdev->memory, (raster << 1) + 1,
+    byte *cmp = gs_alloc_bytes(pdev->memory, cmpsize,
                                "bjc true comp buffer"); /*worst case */
     byte *rowC = dit;                 /*C*/
     byte *rowM = dit + raster;        /*M*/
@@ -851,8 +904,8 @@ bjc_print_page_color(gx_device_printer * pdev, FILE * file)
     char color = 0x10; /* color */
     char ink   = 0x01; /* regular ink type */
     char compress = (ppdev->compress == true ? 0x01 : 0x00); /* compression or not */
-    int x_resolution = pdev->HWResolution[0];
-    int y_resolution = pdev->HWResolution[1];
+    int x_resolution = (int)pdev->HWResolution[0];
+    int y_resolution = (int)pdev->HWResolution[1];
     int length = 0/*x71*/, lm = 0/*x01*/, rm = 0/*x01*/, top = 0/*x50*/;
     byte mask_array[] = { 0xff, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe };
     byte lastmask = mask_array[pdev->width % 8];
