@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* JPEG output driver */
@@ -31,6 +31,7 @@ typedef struct gx_device_jpeg_s {
     int JPEGQ;			/* quality on IJG scale */
     float QFactor;		/* quality per DCTEncode conventions */
     /* JPEGQ overrides QFactor if both are specified. */
+    bool EmbedProfile;
 
     /** 1.0 default 2.0 is twice as big
      */
@@ -80,6 +81,7 @@ const gx_device_jpeg gs_jpeg_device =
                      X_DPI, Y_DPI, 0, 0, 0, 0, 24, jpeg_print_page),
  0,				/* JPEGQ: 0 indicates not specified */
  0.0,				/* QFactor: 0 indicates not specified */
+ 1,                 /* Embed an ICC profile in the output */
  { 1.0, 1.0 },                  /* ViewScale 1 to 1 */
  { 0.0, 0.0 },                  /* translation 0 */
  GX_DOWNSCALER_PARAMS_DEFAULTS
@@ -107,6 +109,7 @@ const gx_device_jpeg gs_jpeggray_device =
                  jpeg_print_page),
  0,				/* JPEGQ: 0 indicates not specified */
  0.0,				/* QFactor: 0 indicates not specified */
+ 1,                 /* Embed an ICC profile in the output */
  { 1.0, 1.0 },                  /* ViewScale 1 to 1 */
  { 0.0, 0.0 },                   /* translation 0 */
  GX_DOWNSCALER_PARAMS_DEFAULTS
@@ -135,6 +138,7 @@ const gx_device_jpeg gs_jpegcmyk_device =
                      X_DPI, Y_DPI, 0, 0, 0, 0, 32, jpeg_print_page),
  0,				/* JPEGQ: 0 indicates not specified */
  0.0,				/* QFactor: 0 indicates not specified */
+ 1,                 /* Embed an ICC profile in the output */
  { 1.0, 1.0 },                  /* ViewScale 1 to 1 */
  { 0.0, 0.0 },                   /* translation 0 */
  GX_DOWNSCALER_PARAMS_DEFAULTS
@@ -172,7 +176,7 @@ jpegcmyk_map_cmyk_color(gx_device * dev, const gx_color_value cv[])
 
 static int
 jpegcmyk_decode_color(gx_device * dev, gx_color_index color,
-                      gx_color_value cv[4])
+                      gx_color_value cv[])
 {
     color = ~color;
     cv[0] = gx_color_value_from_byte(0xff & (color>>24));
@@ -199,6 +203,8 @@ jpeg_get_params(gx_device * dev, gs_param_list * plist)
         ecode = code;
     if ((ecode = param_write_int(plist, "JPEGQ", &jdev->JPEGQ)) < 0)
         code = ecode;
+    if ((ecode = param_write_bool(plist, "EmbedProfile", &jdev->EmbedProfile)) < 0)
+        code = ecode;
     if ((ecode = param_write_float(plist, "QFactor", &jdev->QFactor)) < 0)
         code = ecode;
     float2double = jdev->ViewScale.x;
@@ -228,6 +234,7 @@ jpeg_put_params(gx_device * dev, gs_param_list * plist)
     int jq = jdev->JPEGQ;
     float qf = jdev->QFactor;
     float fparam;
+    bool embed_profile = jdev->EmbedProfile, bparam;
 
     ecode = gx_downscaler_read_params(plist, &jdev->downscale, 0);
 
@@ -301,6 +308,16 @@ jpeg_put_params(gx_device * dev, gs_param_list * plist)
         ecode = code;
         param_signal_error(plist, param_name, code);
     }
+
+    code = param_read_bool(plist, (param_name = "EmbedProfile"), &bparam);
+    if ( code == 0 ) {
+        jdev->EmbedProfile = bparam;
+    }
+    else if ( code < 1 ) {
+        ecode = code;
+        param_signal_error(plist, param_name, code);
+    }
+
     code = gdev_prn_put_params(dev, plist);
     if (code < 0)
         return code;
@@ -396,7 +413,7 @@ jpeg_print_page(gx_device_printer * pdev, gp_file * prn_stream)
 {
     gx_device_jpeg *jdev = (gx_device_jpeg *) pdev;
     gs_memory_t *mem = pdev->memory;
-    int line_size = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
+    size_t line_size = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
     byte *in = gs_alloc_bytes(mem, line_size, "jpeg_print_page(in)");
     jpeg_compress_data *jcdp = gs_alloc_struct_immovable(mem, jpeg_compress_data,
       &st_jpeg_compress_data, "jpeg_print_page(jpeg_compress_data)");
@@ -445,7 +462,7 @@ jpeg_print_page(gx_device_printer * pdev, gp_file * prn_stream)
         pdev->icc_struct->device_profile[GS_DEFAULT_DEVICE_PROFILE] != NULL) {
         cmm_profile_t *icc_profile = pdev->icc_struct->device_profile[GS_DEFAULT_DEVICE_PROFILE];
         if (icc_profile->num_comps == pdev->color_info.num_components &&
-            !(pdev->icc_struct->usefastcolor)) {
+            !(pdev->icc_struct->usefastcolor) && jdev->EmbedProfile) {
             state.icc_profile = icc_profile;
         }
     }
