@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -90,7 +90,7 @@ perm_print_page(gx_device_printer *pdev, gp_file *pstream)
     int y;
     gx_device_perm_t * const dev = (gx_device_perm_t *)pdev;
     int ncomp = dev->num_std_colorant_names;
-    int raw_raster = pdev->width * ncomp;
+    size_t raw_raster = pdev->width * ncomp;
     byte *raw_line = NULL;
     byte *cooked_line = NULL;
     byte *row;
@@ -100,7 +100,7 @@ perm_print_page(gx_device_printer *pdev, gp_file *pstream)
 
     fprintf(pstream, "P6\n%d %d\n255\n", dev->width, dev->height);
     raw_line = gs_alloc_bytes(pdev->memory, raw_raster, "perm_print_page");
-    cooked_line = gs_alloc_bytes(pdev->memory, dev->width * 3, "perm_print_page");
+    cooked_line = gs_alloc_bytes(pdev->memory, (size_t)dev->width * 3, "perm_print_page");
     if (raw_line == NULL || cooked_line == NULL) {
         gs_free_object(pdev->memory, raw_line, "perm_print_page");
         return_error(gs_error_VMerror);
@@ -146,7 +146,7 @@ perm_print_page(gx_device_printer *pdev, gp_file *pstream)
             cooked_line[x * 3 + 1] = g;
             cooked_line[x * 3 + 2] = b;
         }
-        fwrite(cooked_line, 1, dev->width * 3, pstream);
+        gp_fwrite(cooked_line, 1, dev->width * 3, pstream);
     }
     gs_free_object(pdev->memory, cooked_line, "perm_print_page");
     gs_free_object(pdev->memory, raw_line, "perm_print_page");
