@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -106,8 +106,8 @@ static int
 pj_common_print_page(gx_device_printer *pdev, gp_file *prn_stream, int y_origin,
   const char *end_page)
 {
-        int line_size;
-        int data_size;
+        size_t line_size;
+        size_t data_size;
         byte *data = NULL;
         byte *plane_data = NULL;
         byte *temp = NULL;
@@ -145,7 +145,7 @@ pj_common_print_page(gx_device_printer *pdev, gp_file *prn_stream, int y_origin,
         gp_fprintf(prn_stream, "\033*t%dR", X_DPI);
 
         /* set the line width */
-        gp_fprintf(prn_stream, "\033*r%dS", data_size);
+        gp_fprintf(prn_stream, "\033*r%"PRId64"S", data_size);
 
         /* set the number of color planes */
         gp_fprintf(prn_stream, "\033*r%dU", 3);		/* always 3 */
