@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,10 +9,12 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
+#ifndef rinkj_dither_h_INCLUDED
+#define rinkj_dither_h_INCLUDED
 
 /* The dither object abstraction within the Rinkj driver. */
 
@@ -28,3 +30,5 @@ rinkj_dither_line (RinkjDither *self, unsigned char *dst, const unsigned char *s
 
 void
 rinkj_dither_close (RinkjDither *self);
+
+#endif
