@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -822,7 +822,8 @@ s_Bicubic_data_at(stream_Bicubic_state *const ss, int x, int y, int c)
         y = ss->HeightIn - 1;
     y -= ss->y_in;
     idx = ss->l_size * (y < 0 ? 0 : y) +
-        (x < 0 ? 0 : x >= ss->WidthIn ? ss->WidthIn-1 : x) * ss->Colors + c;
+            (size_t)(x < 0 ? 0 : x >= ss->WidthIn ? ss->WidthIn-1 : x) *
+                                                         ss->Colors + c;
     return (idx < ss->d_len) ? ss->data[idx] : 0;
 }
 
@@ -952,7 +953,8 @@ s_compr_chooser_set_dimensions(stream_compr_chooser_state * ss, int width,
     ss->height = height;
     ss->depth = depth;
     ss->bits_per_sample = bits_per_sample;
-    ss->sample = gs_alloc_bytes(ss->memory, width * depth, "s_compr_chooser_set_dimensions");
+    ss->sample = gs_alloc_bytes(ss->memory, (size_t)width * depth,
+                                "s_compr_chooser_set_dimensions");
     if (ss->sample == 0)
         return_error(gs_error_VMerror);
     return 0;
@@ -1155,7 +1157,7 @@ s_compr_chooser__get_choice(stream_compr_chooser_state *ss, bool force)
     if (force) {
         if (ss->gradients > plateaus / 12) /* messenger16.pdf, page 3. */
             return 1; /* photo */
-        else if (plateaus / 5000 >= ss->gradients)
+        else if (plateaus && plateaus / 5000 >= ss->gradients)
             return 2; /* lineart */
     }
     return 0;
@@ -1248,13 +1250,14 @@ s_image_colors_set_mask_colors(stream_image_colors_state * ss, uint *MaskColor)
 /* Set image dimensions. */
 void
 s_image_colors_set_dimensions(stream_image_colors_state * ss,
-                               int width, int height, int depth, int bits_per_sample)
+                              int width, int height, int input_width,
+                              int depth, int bits_per_sample)
 {
     ss->width = width;
     ss->height = height;
     ss->depth = depth;
     ss->bits_per_sample = bits_per_sample;
-    ss->row_bits = bits_per_sample * depth * width;
+    ss->row_bits = bits_per_sample * depth * input_width;
     ss->raster = bitmap_raster(ss->row_bits);
     ss->row_alignment_bytes = 0; /* (ss->raster * 8 - ss->row_bits) / 8) doesn't work. */
 }
