@@ -934,15 +934,15 @@ jbig2_decode_generic_template0_TPGDON(Jbig2Ctx *ctx,
                 uint32_t out_byte = 0;
                 int out_bits_to_go_in_byte = 8;
                 uint8_t *d = &image->data[image->stride * y];
-                uint8_t *pline  = &image->data[image->stride * (y-1)];
-                uint8_t *ppline = &image->data[image->stride * (y-2)];
+                uint8_t *pline  = y > 0 ? &image->data[image->stride * (y-1)] : NULL;
+                uint8_t *ppline = y > 1 ? &image->data[image->stride * (y-2)] : NULL;
                 uint32_t pd = 0;
                 uint32_t ppd = 0;
-                if (y > 0) {
+                if (y > 0 && pline) {
                     pd = (*pline++ << 8);
                     if (GBW > 8)
                         pd |= *pline++;
-                    if (y > 1) {
+                    if (y > 1 && ppline) {
                         ppd = (*ppline++ << 8);
                         if (GBW > 8)
                             ppd |= *ppline++;
@@ -966,9 +966,9 @@ jbig2_decode_generic_template0_TPGDON(Jbig2Ctx *ctx,
                     if (out_bits_to_go_in_byte == 0) {
                         out_bits_to_go_in_byte = 8;
                         *d++ = (uint8_t)out_byte;
-                        if (x+9 < GBW && y > 0) {
+                        if (x+9 < GBW && y > 0 && pline) {
                             pd |= *pline++;
-                            if (y > 1)
+                            if (y > 1 && ppline)
                                 ppd |= *ppline++;
                         }
                     }
@@ -1032,15 +1032,15 @@ jbig2_decode_generic_template0_TPGDON(Jbig2Ctx *ctx,
             uint32_t out_byte = 0;
             int out_bits_to_go_in_byte = 8;
             uint8_t *d = &image->data[image->stride * y];
-            uint8_t *pline  = &image->data[image->stride * (y-1)];
-            uint8_t *ppline = &image->data[image->stride * (y-2)];
+            uint8_t *pline  = y > 0 ? &image->data[image->stride * (y-1)] : NULL;
+            uint8_t *ppline = y > 1 ? &image->data[image->stride * (y-2)] : NULL;
             uint32_t pd = 0;
             uint32_t ppd = 0;
-            if (y > 0) {
+            if (y > 0 && pline) {
                 pd = (*pline++ << 8);
                 if (GBW > 8)
                     pd |= *pline++;
-                if (y > 1) {
+                if (y > 1 && ppline) {
                     ppd = (*ppline++ << 8);
                     if (GBW > 8)
                         ppd |= *ppline++;
@@ -1079,9 +1079,9 @@ jbig2_decode_generic_template0_TPGDON(Jbig2Ctx *ctx,
                 if (out_bits_to_go_in_byte == 0) {
                     out_bits_to_go_in_byte = 8;
                     d++;
-                    if (x+9 < GBW && y > 0) {
+                    if (x+9 < GBW && y > 0 && pline) {
                         pd |= *pline++;
-                        if (y > 1)
+                        if (y > 1 && ppline)
                             ppd |= *ppline++;
                     }
                 }
@@ -1120,15 +1120,15 @@ jbig2_decode_generic_template1_TPGDON(Jbig2Ctx *ctx,
             uint32_t out_byte = 0;
             int out_bits_to_go_in_byte = 8;
             uint8_t *d = &image->data[image->stride * y];
-            uint8_t *pline  = &image->data[image->stride * (y-1)];
-            uint8_t *ppline = &image->data[image->stride * (y-2)];
+            uint8_t *pline  = y > 0 ? &image->data[image->stride * (y-1)] : NULL;
+            uint8_t *ppline = y > 1 ? &image->data[image->stride * (y-2)] : NULL;
             uint32_t pd = 0;
             uint32_t ppd = 0;
-            if (y > 0) {
+            if (y > 0 && pline) {
                 pd = (*pline++ << 8);
                 if (GBW > 8)
                     pd |= *pline++;
-                if (y > 1) {
+                if (y > 1 && ppline) {
                     ppd = (*ppline++ << 8);
                     if (GBW > 8)
                         ppd |= *ppline++;
@@ -1154,9 +1154,9 @@ jbig2_decode_generic_template1_TPGDON(Jbig2Ctx *ctx,
                 if (out_bits_to_go_in_byte == 0) {
                     out_bits_to_go_in_byte = 8;
                     d++;
-                    if (x+9 < GBW && y > 0) {
+                    if (x+9 < GBW && y > 0 && pline) {
                         pd |= *pline++;
-                        if (y > 1)
+                        if (y > 1 && ppline)
                             ppd |= *ppline++;
                     }
                 }
@@ -1195,15 +1195,15 @@ jbig2_decode_generic_template2_TPGDON(Jbig2Ctx *ctx,
             uint32_t out_byte = 0;
             int out_bits_to_go_in_byte = 8;
             uint8_t *d = &image->data[image->stride * y];
-            uint8_t *pline  = &image->data[image->stride * (y-1)];
-            uint8_t *ppline = &image->data[image->stride * (y-2)];
+            uint8_t *pline  =  y > 0 ? &image->data[image->stride * (y-1)] : NULL;
+            uint8_t *ppline = y > 1 ? &image->data[image->stride * (y-2)] : NULL;
             uint32_t pd = 0;
             uint32_t ppd = 0;
-            if (y > 0) {
+            if (y > 0 && pline) {
                 pd = (*pline++ << 8);
                 if (GBW > 8)
                     pd |= *pline++;
-                if (y > 1) {
+                if (y > 1 && ppline) {
                     ppd = (*ppline++ << 8);
                     if (GBW > 8)
                         ppd |= *ppline++;
@@ -1229,9 +1229,9 @@ jbig2_decode_generic_template2_TPGDON(Jbig2Ctx *ctx,
                 if (out_bits_to_go_in_byte == 0) {
                     out_bits_to_go_in_byte = 8;
                     d++;
-                    if (x+9 < GBW && y > 0) {
+                    if (x+9 < GBW && y > 0 && pline) {
                         pd |= *pline++;
-                        if (y > 1)
+                        if (y > 1 && ppline)
                             ppd |= *ppline++;
                     }
                 }
@@ -1270,9 +1270,9 @@ jbig2_decode_generic_template3_TPGDON(Jbig2Ctx *ctx,
             uint32_t out_byte = 0;
             int out_bits_to_go_in_byte = 8;
             uint8_t *d = &image->data[image->stride * y];
-            uint8_t *pline  = &image->data[image->stride * (y-1)];
+            uint8_t *pline  = y > 0 ? &image->data[image->stride * (y-1)] : NULL;
             uint32_t pd = 0;
-            if (y > 0) {
+            if (y > 0 && pline) {
                 pd = (*pline++ << 8);
                 if (GBW > 8)
                     pd |= *pline++;
@@ -1295,7 +1295,7 @@ jbig2_decode_generic_template3_TPGDON(Jbig2Ctx *ctx,
                 if (out_bits_to_go_in_byte == 0) {
                     out_bits_to_go_in_byte = 8;
                     d++;
-                    if (x+9 < GBW && y > 0)
+                    if (x+9 < GBW && y > 0 && pline)
                         pd |= *pline++;
                 }
             }
