@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2021 Artifex Software, Inc.
+/* Copyright (C) 2019-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* code for type 1 font handling */
@@ -117,6 +117,9 @@ pdfi_t1_glyph_data(gs_font_type1 *pfont, gs_glyph glyph, gs_glyph_data_t *pgd)
 
     if (code >= 0) {
         code = pdfi_dict_get_by_key(ctx, pdffont1->CharStrings, glyphname, (pdf_obj **)&charstring);
+        if (code < 0) {
+            code = pdfi_map_glyph_name_via_agl(pdffont1->CharStrings, glyphname, &charstring);
+        }
         if (code >= 0)
             gs_glyph_data_from_bytes(pgd, charstring->data, 0, charstring->length, NULL);
     }
@@ -132,11 +135,17 @@ pdfi_t1_subr_data(gs_font_type1 *pfont, int index, bool global, gs_glyph_data_t
     int code = 0;
     pdf_font_type1 *pdffont1 = (pdf_font_type1 *) pfont->client_data;
 
-    if (global == true || index < 0 || index >= pdffont1->NumSubrs) {
+    if (global == true || index < 0 || index >= (pdffont1->Subrs == NULL ? 0 : pdfi_array_size(pdffont1->Subrs))) {
         code = gs_note_error(gs_error_rangecheck);
     }
     else {
-        gs_glyph_data_from_bytes(pgd, pdffont1->Subrs[index].data, 0, pdffont1->Subrs[index].size, NULL);
+        pdf_string *subr_str = NULL;
+        code = pdfi_array_get_type(pdffont1->ctx, pdffont1->Subrs, index, PDF_STRING, (pdf_obj **)&subr_str);
+        if (code >= 0) {
+            gs_glyph_data_from_bytes(pgd, subr_str->data, 0, subr_str->length, NULL);
+        }
+        /* decrementing is safe here, because the reference in the pdffont1->Subrs will persist */
+        pdfi_countdown(subr_str);
     }
     return code;
 }
@@ -168,11 +177,13 @@ pdfi_t1_seac_data(gs_font_type1 *pfont, int ccode, gs_glyph *pglyph, gs_const_st
             pdfi_countup(glyphname);
             code = pdfi_dict_get_by_key(ctx, pdffont1->CharStrings, glyphname, (pdf_obj **)&charstring);
             pdfi_countdown(glyphname);
-            if (code >= 0)
-                if (pgd != NULL)
+            if (code >= 0) {
+                if (pgd != NULL) {
                     gs_glyph_data_from_bytes(pgd, charstring->data, 0, charstring->length, NULL);
+                }
                 pdfi_countdown(charstring);
             }
+        }
     }
 
     return code;
@@ -368,7 +379,7 @@ pdfi_t1_decode_pfb(pdf_context *ctx, byte *inbuf, int inlen, byte **outbuf, int
     *outbuf = NULL;
     *outlen = 0;
 
-    strm = push_pfb_filter(ctx->memory, inbuf, inbuf + inlen + 1);
+    strm = push_pfb_filter(ctx->memory, inbuf, inbuf + inlen);
     if (strm == NULL) {
         code = gs_note_error(gs_error_VMerror);
     }
@@ -386,7 +397,7 @@ pdfi_t1_decode_pfb(pdf_context *ctx, byte *inbuf, int inlen, byte **outbuf, int
         }
         else {
             d = decodebuf;
-            strm = push_pfb_filter(ctx->memory, inbuf, inbuf + inlen + 1);
+            strm = push_pfb_filter(ctx->memory, inbuf, inbuf + inlen);
             while (1) {
                 c = sgetc(strm);
                 if (c < 0)
@@ -420,8 +431,7 @@ pdfi_alloc_t1_font(pdf_context *ctx, pdf_font_type1 **font, uint32_t obj_num)
 
 #if REFCNT_DEBUG
     t1font->UID = ctx->UID++;
-    dmprintf2(ctx->memory,
-              "Allocated object of type %c with UID %" PRIi64 "\n", t1font->type, t1font->UID);
+    outprintf(ctx->memory, "Allocated object of type %c with UID %" PRIi64 "\n", t1font->type, t1font->UID);
 #endif
 
     pdfi_countup(t1font);
@@ -460,8 +470,8 @@ pdfi_alloc_t1_font(pdf_context *ctx, pdf_font_type1 **font, uint32_t obj_num)
     pfont->id = gs_next_ids(ctx->memory, 1);
     uid_set_UniqueID(&pfont->UID, pfont->id);
 
-    pfont->encoding_index = 1;          /****** WRONG ******/
-    pfont->nearest_encoding_index = 1;          /****** WRONG ******/
+    pfont->encoding_index = ENCODING_INDEX_UNKNOWN;
+    pfont->nearest_encoding_index = ENCODING_INDEX_UNKNOWN;
 
     pfont->client_data = (void *)t1font;
 
@@ -483,7 +493,10 @@ pdfi_t1_font_set_procs(pdf_context *ctx, pdf_font_type1 *font)
     pfont->procs.decode_glyph = pdfi_decode_glyph;
     pfont->procs.define_font = gs_no_define_font;
     pfont->procs.make_font = gs_no_make_font;
-    pfont->procs.font_info = gs_default_font_info;
+
+    font->default_font_info = gs_default_font_info;
+    pfont->procs.font_info = pdfi_default_font_info;
+
     pfont->procs.glyph_info = pdfi_t1_glyph_info;
     pfont->procs.glyph_outline = pdfi_t1_glyph_outline;
     pfont->procs.same_font = gs_default_same_font;
@@ -497,18 +510,34 @@ pdfi_t1_font_set_procs(pdf_context *ctx, pdf_font_type1 *font)
     pfont->data.interpret = gs_type1_interpret;
 }
 
+static inline void
+pdfi_type1_font_priv_defaults(ps_font_interp_private *pfpriv)
+{
+    pfpriv->gsu.gst1.data.lenIV = 4;
+    pfpriv->gsu.gst1.data.ExpansionFactor = 0.06;
+    pfpriv->gsu.gst1.data.BlueShift = 7;
+    pfpriv->gsu.gst1.data.BlueFuzz = 1;
+    pfpriv->gsu.gst1.data.BlueScale = 0.039625;
+    uid_set_invalid(&pfpriv->gsu.gst1.UID);
+}
+
 int
 pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dict, pdf_dict *page_dict, byte *fbuf, int64_t fbuflen, pdf_font **ppdffont)
 {
     int code = 0;
+    double x_scale;
     pdf_obj *fontdesc = NULL;
     pdf_obj *basefont = NULL;
     pdf_obj *mapname = NULL;
     pdf_obj *tmp = NULL;
     pdf_font_type1 *t1f = NULL;
+    pdf_obj *tounicode = NULL;
     ps_font_interp_private fpriv = { 0 };
+    bool key_known;
+    bool force_symbolic = false;
 
-    (void)pdfi_dict_knownget_type(ctx, font_dict, "FontDescriptor", PDF_DICT, &fontdesc);
+    if (font_dict != NULL)
+        (void)pdfi_dict_knownget_type(ctx, font_dict, "FontDescriptor", PDF_DICT, &fontdesc);
 
     if (fbuf[0] == 128 && fbuf[1] == 1) {
         byte *decodebuf = NULL;
@@ -524,17 +553,17 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
     }
 
     if (code >= 0) {
-        fpriv.gsu.gst1.data.lenIV = 4;
+        pdfi_type1_font_priv_defaults(&fpriv);
         code = pdfi_read_ps_font(ctx, font_dict, fbuf, fbuflen, &fpriv);
         gs_free_object(ctx->memory, fbuf, "pdfi_read_type1_font");
 
         /* If we have a full CharStrings dictionary, we probably have enough to make a font */
-        if (code < 0 || (fpriv.u.t1.CharStrings == NULL || fpriv.u.t1.CharStrings->type != PDF_DICT
-            || fpriv.u.t1.CharStrings->entries == 0)) {
+        if (code < 0 || fpriv.u.t1.CharStrings == NULL || pdfi_type_of(fpriv.u.t1.CharStrings) != PDF_DICT
+            || fpriv.u.t1.CharStrings->entries == 0) {
                 code = gs_note_error(gs_error_invalidfont);
                 goto error;
         }
-        code = pdfi_alloc_t1_font(ctx, &t1f, font_dict->object_num);
+        code = pdfi_alloc_t1_font(ctx, &t1f, font_dict != NULL ? font_dict->object_num : 0);
         if (code >= 0) {
             gs_font_type1 *pfont1 = (gs_font_type1 *) t1f->pfont;
 
@@ -550,25 +579,37 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
             if (fpriv.gsu.gst1.UID.id != 0)
                 memcpy(&pfont1->UID, &fpriv.gsu.gst1.UID, sizeof(pfont1->UID));
             fpriv.gsu.gst1.UID.xvalues = NULL; /* In case of error */
-            pfont1->WMode = fpriv.gsu.gst1.WMode;
+            if (fpriv.gsu.gst1.WMode != 0) {
+                if (fpriv.gsu.gst1.WMode != 1)
+                    pdfi_set_warning(ctx, 0, NULL, W_PDF_BAD_WMODE, "pdfi_read_type1_font", NULL);
+                pfont1->WMode = 1;
+            }
+            else
+                pfont1->WMode = 0;
             pfont1->PaintType = fpriv.gsu.gst1.PaintType;
             pfont1->StrokeWidth = fpriv.gsu.gst1.StrokeWidth;
 
-            t1f->object_num = font_dict->object_num;
-            t1f->generation_num = font_dict->generation_num;
+            if (font_dict != NULL) {
+                t1f->object_num = font_dict->object_num;
+                t1f->generation_num = font_dict->generation_num;
+                t1f->indirect_num = font_dict->indirect_num;
+                t1f->indirect_gen = font_dict->indirect_gen;
+            }
+
             t1f->PDF_font = font_dict;
             pdfi_countup(font_dict);
-            t1f->BaseFont = basefont;
-            pdfi_countup(basefont);
             t1f->FontDescriptor = (pdf_dict *) fontdesc;
             pdfi_countup(fontdesc);
             t1f->Name = mapname;
             pdfi_countup(mapname);
 
             /* We want basefont, but we can live without it */
-            (void)pdfi_dict_knownget_type(ctx, font_dict, "BaseFont", PDF_NAME, &basefont);
+            if (font_dict != NULL) {
+                (void)pdfi_dict_knownget_type(ctx, font_dict, "BaseFont", PDF_NAME, &basefont);
+                t1f->BaseFont = basefont;
+                pdfi_countup(basefont);
+            }
 
-            t1f->descflags = 0;
             if (t1f->FontDescriptor != NULL) {
                 code = pdfi_dict_get_int(ctx, t1f->FontDescriptor, "Flags", &t1f->descflags);
                 if (code >= 0) {
@@ -581,96 +622,63 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
             }
 
             if (pdfi_font_known_symbolic(basefont)) {
+                force_symbolic = true;
                 t1f->descflags |= 4;
             }
 
-            code = pdfi_dict_knownget_type(ctx, font_dict, "FirstChar", PDF_INT, &tmp);
-            if (code == 1) {
-                t1f->FirstChar = ((pdf_num *) tmp)->value.i;
-                pdfi_countdown(tmp);
-                tmp = NULL;
+            if (ctx->args.ignoretounicode != true && font_dict != NULL) {
+                code = pdfi_dict_get(ctx, font_dict, "ToUnicode", (pdf_obj **)&tounicode);
+                if (code >= 0 && pdfi_type_of(tounicode) == PDF_STREAM) {
+                    pdf_cmap *tu = NULL;
+                    code = pdfi_read_cmap(ctx, tounicode, &tu);
+                    pdfi_countdown(tounicode);
+                    tounicode = (pdf_obj *)tu;
+                }
+                if (code < 0 || (tounicode != NULL && pdfi_type_of(tounicode) != PDF_CMAP)) {
+                    pdfi_countdown(tounicode);
+                    tounicode = NULL;
+                    code = 0;
+                }
             }
             else {
-                t1f->FirstChar = 0;
+                tounicode = NULL;
             }
-            code = pdfi_dict_knownget_type(ctx, font_dict, "LastChar", PDF_INT, &tmp);
+            t1f->ToUnicode = tounicode;
+            tounicode = NULL;
+
+            pdfi_font_set_first_last_char(ctx, font_dict, (pdf_font *)t1f);
+
+            /* Widths are defined assuming a 1000x1000 design grid, but we apply
+             * them in font space - so undo the 1000x1000 scaling, and apply
+             * the inverse of the font's x scaling
+             */
+            x_scale = 0.001 / hypot(pfont1->FontMatrix.xx, pfont1->FontMatrix.xy);
+
+            /* ignore errors with widths... for now */
+            if (font_dict != NULL)
+                (void)pdfi_font_create_widths(ctx, font_dict, (pdf_font*)t1f, x_scale);
+
+            if (font_dict != NULL)
+                code = pdfi_dict_knownget(ctx, font_dict, "Encoding", &tmp);
+            else
+                code = gs_error_undefined;
             if (code == 1) {
-                t1f->LastChar = ((pdf_num *) tmp)->value.i;
-                pdfi_countdown(tmp);
-                tmp = NULL;
-            }
-            else {
-                t1f->LastChar = 255;
-            }
-
-            t1f->fake_glyph_names = (gs_string *) gs_alloc_bytes(ctx->memory, t1f->LastChar * sizeof(gs_string), "pdfi_read_type1_font: fake_glyph_names");
-            if (!t1f->fake_glyph_names) {
-                code = gs_note_error(gs_error_VMerror);
-                goto error;
-            }
-            memset(t1f->fake_glyph_names, 0x00, t1f->LastChar * sizeof(gs_string));
-
-            code = pdfi_dict_knownget_type(ctx, font_dict, "Widths", PDF_ARRAY, &tmp);
-            if (code > 0) {
-                int i;
-                double x_scale;
-                int num_chars = t1f->LastChar - t1f->FirstChar + 1;
-
-                if (num_chars == pdfi_array_size((pdf_array *) tmp)) {
-                    t1f->Widths = (double *)gs_alloc_bytes(ctx->memory, sizeof(double) * num_chars, "Type 1 font Widths array");
-                    if (t1f->Widths == NULL) {
-                        code = gs_note_error(gs_error_VMerror);
-                        goto error;
-                    }
-
-                    /* Widths are defined assuming a 1000x1000 design grid, but we apply
-                     * them in font space - so undo the 1000x1000 scaling, and apply
-                     * the inverse of the font's x scaling
-                     */
-                    x_scale = 0.001 / hypot(pfont1->FontMatrix.xx, pfont1->FontMatrix.xy);
-
-                    memset(t1f->Widths, 0x00, sizeof(double) * num_chars);
-                    for (i = 0; i < num_chars; i++) {
-                        code = pdfi_array_get_number(ctx, (pdf_array *) tmp, (uint64_t) i, &t1f->Widths[i]);
-                        if (code < 0)
-                            goto error;
-                        t1f->Widths[i] *= x_scale;
-                    }
+                if (pdfi_type_of(tmp) == PDF_NAME && force_symbolic == true) {
+                    t1f->Encoding = fpriv.u.t1.Encoding;
+                    pdfi_countup(t1f->Encoding);
+                }
+                else if (pdfi_type_of(tmp) == PDF_DICT && (t1f->descflags & 4) != 0) {
+                    code = pdfi_create_Encoding(ctx, (pdf_font *)t1f, tmp, (pdf_obj *)fpriv.u.t1.Encoding, (pdf_obj **) & t1f->Encoding);
+                    if (code >= 0)
+                        code = 1;
                 }
                 else {
-                    t1f->Widths = NULL;
-                }
-            }
-            pdfi_countdown(tmp);
-            tmp = NULL;
-
-            code = pdfi_dict_knownget(ctx, font_dict, "Encoding", &tmp);
-            if (code == 1) {
-                if ((tmp->type == PDF_NAME || tmp->type == PDF_DICT) && (t1f->descflags & 4) == 0) {
-                    code = pdfi_create_Encoding(ctx, tmp, NULL, (pdf_obj **) & t1f->Encoding);
+                    code = pdfi_create_Encoding(ctx, (pdf_font *)t1f, tmp, NULL, (pdf_obj **) & t1f->Encoding);
                     if (code >= 0)
                         code = 1;
                 }
-                else if (tmp->type == PDF_DICT && (t1f->descflags & 4) != 0) {
-                    code = pdfi_create_Encoding(ctx, tmp, (pdf_obj *)fpriv.u.t1.Encoding, (pdf_obj **) & t1f->Encoding);
-                    if (code >= 0)
-                        code = 1;
-                }
-                else
-                    code = gs_error_undefined;
                 pdfi_countdown(tmp);
                 tmp = NULL;
-                if (code == 1) {
-                    /* Since the underlying font stream can be shared between font descriptors,
-                       and the font descriptors can be shared between font objects, if we change
-                       the encoding, we can't share cached glyphs with other instances of this
-                       underlying font, so invalidate the UniqueID/XUID so the glyph cache won't
-                       try.
-                    */
-                    if (uid_is_XUID(&t1f->pfont->UID))
-                        uid_free(&t1f->pfont->UID, t1f->pfont->memory, "pdfi_read_type1_font");
-                    uid_set_invalid(&t1f->pfont->UID);
-                }
             }
             else {
                 pdfi_countdown(tmp);
@@ -682,22 +690,33 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
                 t1f->Encoding = fpriv.u.t1.Encoding;
                 pdfi_countup(t1f->Encoding);
             }
+            /* Since the underlying font stream can be shared between font descriptors,
+               and the font descriptors can be shared between font objects, if we change
+               the encoding, we can't share cached glyphs with other instances of this
+               underlying font, so invalidate the UniqueID/XUID so the glyph cache won't
+               try.
+            */
+            if (uid_is_XUID(&t1f->pfont->UID))
+                uid_free(&t1f->pfont->UID, t1f->pfont->memory, "pdfi_read_type1_font");
+            uid_set_invalid(&t1f->pfont->UID);
 
-            code = pdfi_dict_knownget(ctx, font_dict, "ToUnicode", &tmp);
-            if (code == 1) {
-                t1f->ToUnicode = tmp;
-                tmp = NULL;
-            }
-            else {
-                t1f->ToUnicode = NULL;
-            }
             t1f->CharStrings = fpriv.u.t1.CharStrings;
             pdfi_countup(t1f->CharStrings);
             pdfi_patch_charstrings_dict(t1f->CharStrings);
 
             t1f->Subrs = fpriv.u.t1.Subrs;
             fpriv.u.t1.Subrs = NULL;
-            t1f->NumSubrs = fpriv.u.t1.NumSubrs;
+
+            t1f->copyright = fpriv.u.t1.copyright;
+            t1f->notice = fpriv.u.t1.notice;
+            t1f->fullname = fpriv.u.t1.fullname;
+            t1f->familyname = fpriv.u.t1.familyname;
+            fpriv.u.t1.copyright = fpriv.u.t1.notice = fpriv.u.t1.fullname = fpriv.u.t1.familyname = NULL;
+
+            code = pdfi_font_generate_pseudo_XUID(ctx, font_dict, t1f->pfont);
+            if (code < 0) {
+                goto error;
+            }
 
             t1f->blenddesignpositions = fpriv.u.t1.blenddesignpositions;
             pdfi_countup(t1f->blenddesignpositions);
@@ -708,6 +727,22 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
             t1f->blendaxistypes = fpriv.u.t1.blendaxistypes;
             pdfi_countup(t1f->blendaxistypes);
 
+            key_known = false;
+            if (t1f->FontDescriptor != NULL) {
+                 code = pdfi_dict_known(ctx, t1f->FontDescriptor, "FontFile", &key_known);
+                 if (code < 0 || key_known == false) {
+                     code = pdfi_dict_known(ctx, t1f->FontDescriptor, "FontFile2", &key_known);
+                     if (code < 0 || key_known == false) {
+                         code = pdfi_dict_known(ctx, t1f->FontDescriptor, "FontFile3", &key_known);
+                         if (code < 0) {
+                             key_known = false;
+                         }
+                     }
+                 }
+            }
+            t1f->pfont->is_resource = (key_known == false);
+
+            pdfi_font_set_orig_fonttype(ctx, (pdf_font *)t1f);
             code = gs_definefont(ctx->font_dir, (gs_font *) t1f->pfont);
             if (code < 0) {
                 goto error;
@@ -719,9 +754,7 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
             }
             /* object_num can be zero if the dictionary was defined inline */
             if (t1f->object_num != 0) {
-                code = replace_cache_entry(ctx, (pdf_obj *) t1f);
-                if (code < 0)
-                    goto error;
+                (void)replace_cache_entry(ctx, (pdf_obj *) t1f);
             }
             *ppdffont = (pdf_font *) t1f;
         }
@@ -730,6 +763,7 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
   error:
     pdfi_countdown(fontdesc);
     pdfi_countdown(basefont);
+    pdfi_countdown(tounicode);
     pdfi_countdown(mapname);
     pdfi_countdown(tmp);
     pdfi_countdown(fpriv.u.t1.Encoding);
@@ -738,32 +772,230 @@ pdfi_read_type1_font(pdf_context *ctx, pdf_dict *font_dict, pdf_dict *stream_dic
     pdfi_countdown(fpriv.u.t1.blenddesignmap);
     pdfi_countdown(fpriv.u.t1.blendfontbbox);
     pdfi_countdown(fpriv.u.t1.blendaxistypes);
+    pdfi_countdown(fpriv.u.t1.Subrs);
+    pdfi_countdown(fpriv.u.t1.copyright);
+    pdfi_countdown(fpriv.u.t1.notice);
+    pdfi_countdown(fpriv.u.t1.fullname);
+    pdfi_countdown(fpriv.u.t1.familyname);
     if (fpriv.gsu.gst1.UID.xvalues != NULL) {
         gs_free_object(ctx->memory, fpriv.gsu.gst1.UID.xvalues, "pdfi_read_type1_font(xuid)");
     }
-    if (fpriv.u.t1.Subrs) {
-        int i;
 
-        for (i = 0; i < fpriv.u.t1.NumSubrs; i++) {
-            gs_free_object(ctx->memory, fpriv.u.t1.Subrs[i].data, "Subrs[i]");
-        }
-        gs_free_object(ctx->memory, fpriv.u.t1.Subrs, "Subrs");
-    }
     if (code < 0) {
+        tmp = NULL;
+        if (font_dict != NULL) {
+            if (pdfi_dict_get(ctx, font_dict, ".Path", &tmp) >= 0)
+            {
+                char fname[gp_file_name_sizeof + 1];
+                pdf_string *fobj = (pdf_string *)tmp;
+
+                memcpy(fname, fobj->data, fobj->length > gp_file_name_sizeof ? gp_file_name_sizeof : fobj->length);
+                fname[fobj->length > gp_file_name_sizeof ? gp_file_name_sizeof : fobj->length] = '\0';
+
+                (void)pdfi_set_error_var(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_type1_font", "Error reading Type 1 font file %s\n", fname);
+            }
+            else {
+                (void)pdfi_set_error_var(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_type1_font", "Error reading embedded Type 1 font object %u\n", font_dict->object_num);
+            }
+        }
+        else {
+            pdfi_set_error(ctx, code, NULL, E_PDF_BADSTREAM, "pdfi_read_truetype_font", "Error reading font\n");
+        }
+        pdfi_countdown(tmp);
         pdfi_countdown(t1f);
     }
     return code;
 }
 
+int
+pdfi_copy_type1_font(pdf_context *ctx, pdf_font *spdffont, pdf_dict *font_dict, pdf_font **tpdffont)
+{
+    int code = 0;
+    pdf_font_type1 *font = NULL;
+    gs_font_type1 *spfont1 = (gs_font_type1 *) spdffont->pfont;
+    gs_font_type1 *dpfont1;
+    gs_id t_id;
+    pdf_obj *tmp;
+    bool force_symbolic = false;
+
+    if (font_dict == NULL)
+        return_error(gs_error_invalidfont);
+
+    code = pdfi_alloc_t1_font(ctx, &font, font_dict->object_num);
+    if (code < 0)
+        return code;
+    dpfont1 = (gs_font_type1 *) font->pfont;
+
+    t_id = dpfont1->id;
+    memcpy(dpfont1, spfont1, sizeof(gs_font_type1));
+    dpfont1->id = t_id;
+    dpfont1->FAPI = NULL;
+    dpfont1->FAPI_font_data = NULL;
+
+    memcpy(font, spdffont, sizeof(pdf_font_type1));
+    font->pfont = (gs_font_base *)dpfont1;
+    font->refcnt = 1;
+    dpfont1->client_data = (void *)font;
+    font->filename = NULL;
+
+    dpfont1->notify_list.memory = NULL;
+    dpfont1->notify_list.first = NULL;
+    gs_notify_init(&dpfont1->notify_list, dpfont1->memory);
+
+    font->PDF_font = font_dict;
+    font->object_num = font_dict->object_num;
+    font->generation_num = font_dict->generation_num;
+    pdfi_countup(font->PDF_font);
+
+    /* We want basefont and descriptor, but we can live without them */
+    font->BaseFont = NULL;
+    code = pdfi_dict_knownget_type(ctx, font_dict, "BaseFont", PDF_NAME, &font->BaseFont);
+    if (code < 0) {
+        pdfi_countdown(font->BaseFont);
+        font->BaseFont = NULL;
+    }
+    font->FontDescriptor = NULL;
+    code = pdfi_dict_knownget_type(ctx, font_dict, "FontDescriptor", PDF_DICT, (pdf_obj **)&font->FontDescriptor);
+    if (code < 0) {
+        pdfi_countdown(font->FontDescriptor);
+        font->FontDescriptor = NULL;
+    }
+
+    pdfi_countup(font->Name);
+    pdfi_countup(font->CharStrings);
+    pdfi_countup(font->blenddesignpositions);
+    pdfi_countup(font->blenddesignmap);
+    pdfi_countup(font->blendfontbbox);
+    pdfi_countup(font->blendaxistypes);
+    pdfi_countup(font->Subrs);
+    pdfi_countup(font->copyright);
+    pdfi_countup(font->notice);
+    pdfi_countup(font->fullname);
+    pdfi_countup(font->familyname);
+
+    if (font->BaseFont != NULL && ((pdf_name *)font->BaseFont)->length <= gs_font_name_max - 1) {
+        memcpy(dpfont1->key_name.chars, ((pdf_name *)font->BaseFont)->data, ((pdf_name *)font->BaseFont)->length);
+        dpfont1->key_name.size = ((pdf_name *)font->BaseFont)->length;
+        dpfont1->key_name.chars[dpfont1->key_name.size] = '\0';
+        memcpy(dpfont1->font_name.chars, ((pdf_name *)font->BaseFont)->data, ((pdf_name *)font->BaseFont)->length);
+        dpfont1->font_name.size = ((pdf_name *)font->BaseFont)->length;
+        dpfont1->font_name.chars[dpfont1->font_name.size] = '\0';
+    }
+
+    font->Encoding = NULL;
+    font->ToUnicode = NULL;
+    font->Widths = NULL;
+
+    pdfi_font_set_first_last_char(ctx, font_dict, (pdf_font *)font);
+    (void)pdfi_font_create_widths(ctx, font_dict, (pdf_font *)font, (double)(0.001 / hypot(dpfont1->FontMatrix.xx, dpfont1->FontMatrix.xy)));
+
+    font->descflags = 0;
+    if (font->FontDescriptor != NULL) {
+        code = pdfi_dict_get_int(ctx, font->FontDescriptor, "Flags", &font->descflags);
+        if (code >= 0) {
+            /* If both the symbolic and non-symbolic flag are set,
+               believe that latter.
+             */
+            if ((font->descflags & 32) != 0)
+                font->descflags = (font->descflags & ~4);
+        }
+    }
+
+    if (pdfi_font_known_symbolic(font->BaseFont)) {
+        force_symbolic = true;
+        font->descflags |= 4;
+    }
+
+    tmp = NULL;
+    code = pdfi_dict_knownget(ctx, font_dict, "Encoding", &tmp);
+    if (code == 1) {
+        if (pdfi_type_of(tmp) == PDF_NAME && force_symbolic == true) {
+            font->Encoding = spdffont->Encoding;
+            pdfi_countup(font->Encoding);
+        }
+        else if (pdfi_type_of(tmp) == PDF_DICT && (font->descflags & 4) != 0) {
+            code = pdfi_create_Encoding(ctx, (pdf_font *)font, tmp, (pdf_obj *)spdffont->Encoding, (pdf_obj **) &font->Encoding);
+            if (code >= 0)
+                code = 1;
+        }
+        else {
+            code = pdfi_create_Encoding(ctx, (pdf_font *)font, tmp, NULL, (pdf_obj **) & font->Encoding);
+            if (code >= 0)
+                code = 1;
+        }
+        pdfi_countdown(tmp);
+        tmp = NULL;
+    }
+    else {
+        pdfi_countdown(tmp);
+        tmp = NULL;
+        code = 0;
+    }
+
+    if (code <= 0) {
+        font->Encoding = spdffont->Encoding;
+        pdfi_countup(font->Encoding);
+    }
+
+    code = uid_copy(&font->pfont->UID, font->pfont->memory, "pdfi_copy_type1_font");
+    if (code < 0) {
+        uid_set_invalid(&font->pfont->UID);
+    }
+    if (spdffont->filename == NULL) {
+        code = pdfi_font_generate_pseudo_XUID(ctx, font_dict, font->pfont);
+        if (code < 0) {
+            goto error;
+        }
+    }
+
+    if (ctx->args.ignoretounicode != true) {
+        code = pdfi_dict_get(ctx, font_dict, "ToUnicode", (pdf_obj **)&tmp);
+        if (code >= 0 && pdfi_type_of(tmp) == PDF_STREAM) {
+            pdf_cmap *tu = NULL;
+            code = pdfi_read_cmap(ctx, tmp, &tu);
+            pdfi_countdown(tmp);
+            tmp = (pdf_obj *)tu;
+        }
+        if (code < 0 || (tmp != NULL && pdfi_type_of(tmp) != PDF_CMAP)) {
+            pdfi_countdown(tmp);
+            tmp = NULL;
+            code = 0;
+        }
+    }
+    else {
+        tmp = NULL;
+    }
+    font->ToUnicode = tmp;
+
+    pdfi_font_set_orig_fonttype(ctx, (pdf_font *)font);
+    code = gs_definefont(ctx->font_dir, (gs_font *) font->pfont);
+    if (code < 0) {
+        goto error;
+    }
+
+    code = pdfi_fapi_passfont((pdf_font *) font, 0, NULL, NULL, NULL, 0);
+    if (code < 0) {
+        goto error;
+    }
+    /* object_num can be zero if the dictionary was defined inline */
+    if (font->object_num != 0) {
+        (void)replace_cache_entry(ctx, (pdf_obj *) font);
+    }
+
+    *tpdffont = (pdf_font *)font;
+
+error:
+    if (code < 0)
+        pdfi_countdown(font);
+
+    return code;
+}
+
 int
 pdfi_free_font_type1(pdf_obj *font)
 {
     pdf_font_type1 *t1f = (pdf_font_type1 *) font;
-    int i;
 
-    if (t1f->pfont->UID.xvalues != NULL) {
-        gs_free_object(OBJ_MEMORY(font), t1f->pfont->UID.xvalues, "pdfi_free_font_type1(xuid)");
-    }
     gs_free_object(OBJ_MEMORY(font), t1f->pfont, "Free Type 1 gs_font");
 
     pdfi_countdown(t1f->PDF_font);
@@ -777,20 +1009,13 @@ pdfi_free_font_type1(pdf_obj *font)
     pdfi_countdown(t1f->blenddesignmap);
     pdfi_countdown(t1f->blendfontbbox);
     pdfi_countdown(t1f->blendaxistypes);
+    pdfi_countdown(t1f->Subrs);
+    pdfi_countdown(t1f->filename);
+    pdfi_countdown(t1f->copyright);
+    pdfi_countdown(t1f->notice);
+    pdfi_countdown(t1f->fullname);
+    pdfi_countdown(t1f->familyname);
 
-    if (t1f->fake_glyph_names != NULL) {
-        for (i = 0; i < t1f->LastChar; i++) {
-            if (t1f->fake_glyph_names[i].data != NULL)
-                gs_free_object(OBJ_MEMORY(font), t1f->fake_glyph_names[i].data, "Type 1 fake_glyph_name");
-        }
-        gs_free_object(OBJ_MEMORY(font), t1f->fake_glyph_names, "Type 1 fake_glyph_names");
-    }
-    if (t1f->NumSubrs > 0 && t1f->Subrs != NULL) {
-        for (i = 0; i < t1f->NumSubrs; i++) {
-            gs_free_object(OBJ_MEMORY(font), t1f->Subrs[i].data, "Type 1 Subr");
-        }
-        gs_free_object(OBJ_MEMORY(font), t1f->Subrs, "Type 1 Subrs");
-    }
     gs_free_object(OBJ_MEMORY(font), t1f->Widths, "Free Type 1 fontWidths");
     gs_free_object(OBJ_MEMORY(font), t1f, "Free Type 1 font");
     return 0;
