@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2025 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* prevent gp.h redefining sprintf */
@@ -40,7 +40,7 @@
 #endif
 
 GSDLL gsdll;
-void *instance;
+void *instance = NULL;
 #ifndef METRO
 BOOL quitnow = FALSE;
 HANDLE hthread;
@@ -109,6 +109,44 @@ gsdll_stderr(void *instance, const char *str, int len)
     return len;
 }
 
+/* make a file readable, for drag'n'drop */
+/* In this version (for the console app) TCHAR is just char */
+int dwmain_add_file_control_path(const TCHAR *pathfile)
+{
+    int i, code;
+    char *p = (char *)pathfile;
+
+    for (i = 0; i < strlen(p); i++) {
+        if (p[i] == '\\') {
+            p[i] = '/';
+        }
+    }
+    code = gsdll.add_control_path(instance, GS_PERMIT_FILE_READING, (const char *)p);
+    for (i = 0; i < strlen(p); i++) {
+        if (p[i] == '/') {
+            p[i] = '\\';
+        }
+    }
+    return code;
+}
+void dwmain_remove_file_control_path(const TCHAR *pathfile)
+{
+    int i;
+    char *p = (char *)pathfile;
+
+    for (i = 0; i < strlen(p); i++) {
+        if (p[i] == '\\') {
+            p[i] = '/';
+        }
+    }
+    gsdll.remove_control_path(instance, GS_PERMIT_FILE_READING, (const char *)p);
+    for (i = 0; i < strlen(p); i++) {
+        if (p[i] == '/') {
+            p[i] = '\\';
+        }
+    }
+}
+
 /* stdio functions - versions that translate to/from utf-8 */
 static int GSDLLCALL
 gsdll_stdin_utf8(void *instance, char *buf, int len)
@@ -130,7 +168,7 @@ gsdll_stdin_utf8(void *instance, char *buf, int len)
             /* Previously the code here has always just checked for whether
              * _read returns <= 0 to see whether we should exit. According
              * to the docs -1 means error, 0 means EOF. Unfortunately,
-             * building using VS2015 there appears to be a bug in the 
+             * building using VS2015 there appears to be a bug in the
              * runtime, whereby a line with a single return on it (on an
              * ANSI encoded Text file at least) causes a 0 return value.
              * We hack around this by second guessing the code. We clear
@@ -653,7 +691,7 @@ static int main_utf8(int argc, char *argv[])
         sprintf(dformat, "-dDisplayFormat=%d", format);
     }
     nargc = argc + 2;
-    nargv = (char **)malloc(nargc * sizeof(char *));
+    nargv = (char **)malloc((size_t)nargc * sizeof(char *));
     if (nargv == NULL) {
         fprintf(stderr, "Malloc failure!\n");
     } else {
@@ -721,14 +759,14 @@ int wmain(int argc, wchar_t *argv[], wchar_t *envp[]) {
     if (nargv == NULL)
         goto err;
     for (i=0; i < argc; i++) {
-        nargv[i] = malloc(wchar_to_utf8(NULL, argv[i]));
+        nargv[i] = malloc(gp_uint16_to_utf8(NULL, argv[i]));
         if (nargv[i] == NULL)
             goto err;
-        (void)wchar_to_utf8(nargv[i], argv[i]);
+        (void)gp_uint16_to_utf8(nargv[i], argv[i]);
     }
     code = main_utf8(argc, nargv);
 
-    if (0) {
+     if (0) {
 err:
         fprintf(stderr,
                 "Ghostscript failed to initialise due to malloc failure\n");
