@@ -24,63 +24,62 @@
 #ifndef TESSERACT_CCSTRUCT_MATRIX_H_
 #define TESSERACT_CCSTRUCT_MATRIX_H_
 
-#include "errcode.h"    // for ASSERT_HOST
-#include "helpers.h"    // for ReverseN, ClipToRange
-#include "kdpair.h"     // for KDPairInc
-#include "points.h"     // for ICOORD
+#include "errcode.h" // for ASSERT_HOST
+#include "helpers.h" // for ReverseN, ClipToRange
+#include "kdpair.h"  // for KDPairInc
+#include "points.h"  // for ICOORD
 
-#include "serialis.h"   // for TFile
+#include "serialis.h" // for TFile
 
-#include <algorithm>    // for max, min
-#include <cmath>        // for sqrt, fabs, isfinite
-#include <cstdint>      // for int32_t
-#include <cstdio>       // for FILE
-#include <cstring>      // for memcpy
+#include <algorithm> // for max, min
+#include <cmath>     // for sqrt, fabs, isfinite
+#include <cstdint>   // for int32_t
+#include <cstdio>    // for FILE
+#include <cstring>   // for memcpy
 
 namespace tesseract {
 
 class BLOB_CHOICE_LIST;
 class UNICHARSET;
 
-#define NOT_CLASSIFIED static_cast<BLOB_CHOICE_LIST*>(nullptr)
+#define NOT_CLASSIFIED static_cast<BLOB_CHOICE_LIST *>(nullptr)
 
 // A generic class to hold a 2-D matrix with entries of type T, but can also
 // act as a base class for other implementations, such as a triangular or
 // banded matrix.
 template <class T>
 class GENERIC_2D_ARRAY {
- public:
+public:
   // Initializes the array size, and empty element, but cannot allocate memory
   // for the subclasses or initialize because calls to the num_elements
   // member will be routed to the base class implementation. Subclasses can
   // either pass the memory in, or allocate after by calling Resize().
-  GENERIC_2D_ARRAY(int dim1, int dim2, const T& empty, T* array)
-    : empty_(empty), dim1_(dim1), dim2_(dim2), array_(array)  {
+  GENERIC_2D_ARRAY(int dim1, int dim2, const T &empty, T *array)
+      : empty_(empty), dim1_(dim1), dim2_(dim2), array_(array) {
     size_allocated_ = dim1 * dim2;
   }
   // Original constructor for a full rectangular matrix DOES allocate memory
   // and initialize it to empty.
-  GENERIC_2D_ARRAY(int dim1, int dim2, const T& empty)
-    : empty_(empty), dim1_(dim1), dim2_(dim2)  {
+  GENERIC_2D_ARRAY(int dim1, int dim2, const T &empty) : empty_(empty), dim1_(dim1), dim2_(dim2) {
     int new_size = dim1 * dim2;
     array_ = new T[new_size];
     size_allocated_ = new_size;
-    for (int i = 0; i < size_allocated_; ++i)
+    for (int i = 0; i < size_allocated_; ++i) {
       array_[i] = empty_;
+    }
   }
   // Default constructor for array allocation. Use Resize to set the size.
   GENERIC_2D_ARRAY()
-    : array_(nullptr), empty_(static_cast<T>(0)), dim1_(0), dim2_(0),
-      size_allocated_(0) {
-  }
-  GENERIC_2D_ARRAY(const GENERIC_2D_ARRAY<T>& src)
-    : array_(nullptr), empty_(static_cast<T>(0)), dim1_(0), dim2_(0),
-      size_allocated_(0) {
+      : array_(nullptr), empty_(static_cast<T>(0)), dim1_(0), dim2_(0), size_allocated_(0) {}
+  GENERIC_2D_ARRAY(const GENERIC_2D_ARRAY<T> &src)
+      : array_(nullptr), empty_(static_cast<T>(0)), dim1_(0), dim2_(0), size_allocated_(0) {
     *this = src;
   }
-  virtual ~GENERIC_2D_ARRAY() { delete[] array_; }
+  virtual ~GENERIC_2D_ARRAY() {
+    delete[] array_;
+  }
 
-  void operator=(const GENERIC_2D_ARRAY<T>& src) {
+  void operator=(const GENERIC_2D_ARRAY<T> &src) {
     ResizeNoInit(src.dim1(), src.dim2());
     int size = num_elements();
     if (size > 0) {
@@ -95,18 +94,20 @@ class GENERIC_2D_ARRAY {
   void ResizeNoInit(int size1, int size2, int pad = 0) {
     int new_size = size1 * size2 + pad;
     if (new_size > size_allocated_) {
-      delete [] array_;
+      delete[] array_;
       array_ = new T[new_size];
       size_allocated_ = new_size;
     }
     dim1_ = size1;
     dim2_ = size2;
     // Fill the padding data so it isn't uninitialized.
-    for (int i = size1 * size2; i < new_size; ++i) array_[i] = empty_;
+    for (int i = size1 * size2; i < new_size; ++i) {
+      array_[i] = empty_;
+    }
   }
 
   // Reallocate the array to the given size. Does not keep old data.
-  void Resize(int size1, int size2, const T& empty) {
+  void Resize(int size1, int size2, const T &empty) {
     empty_ = empty;
     ResizeNoInit(size1, size2);
     Clear();
@@ -116,7 +117,7 @@ class GENERIC_2D_ARRAY {
   void ResizeWithCopy(int size1, int size2) {
     if (size1 != dim1_ || size2 != dim2_) {
       int new_size = size1 * size2;
-      T* new_array = new T[new_size];
+      T *new_array = new T[new_size];
       for (int col = 0; col < size1; ++col) {
         for (int row = 0; row < size2; ++row) {
           int old_index = col * dim2() + row;
@@ -139,22 +140,31 @@ class GENERIC_2D_ARRAY {
   // Sets all the elements of the array to the empty value.
   void Clear() {
     int total_size = num_elements();
-    for (int i = 0; i < total_size; ++i)
+    for (int i = 0; i < total_size; ++i) {
       array_[i] = empty_;
+    }
   }
 
   // Writes to the given file. Returns false in case of error.
   // Only works with bitwise-serializeable types!
-  bool Serialize(FILE* fp) const {
-    if (!SerializeSize(fp)) return false;
-    if (!tesseract::Serialize(fp, &empty_)) return false;
+  bool Serialize(FILE *fp) const {
+    if (!SerializeSize(fp)) {
+      return false;
+    }
+    if (!tesseract::Serialize(fp, &empty_)) {
+      return false;
+    }
     int size = num_elements();
     return tesseract::Serialize(fp, &array_[0], size);
   }
 
-  bool Serialize(TFile* fp) const {
-    if (!SerializeSize(fp)) return false;
-    if (!fp->Serialize(&empty_)) return false;
+  bool Serialize(TFile *fp) const {
+    if (!SerializeSize(fp)) {
+      return false;
+    }
+    if (!fp->Serialize(&empty_)) {
+      return false;
+    }
     int size = num_elements();
     return fp->Serialize(&array_[0], size);
   }
@@ -162,33 +172,47 @@ class GENERIC_2D_ARRAY {
   // Reads from the given file. Returns false in case of error.
   // Only works with bitwise-serializeable types!
   // If swap is true, assumes a big/little-endian swap is needed.
-  bool DeSerialize(bool swap, FILE* fp) {
-    if (!DeSerializeSize(swap, fp)) return false;
-    if (!tesseract::DeSerialize(fp, &empty_)) return false;
-    if (swap) ReverseN(&empty_, sizeof(empty_));
-    int size = num_elements();
-    if (!tesseract::DeSerialize(fp, &array_[0], size)) return false;
+  bool DeSerialize(bool swap, FILE *fp) {
+    if (!DeSerializeSize(swap, fp)) {
+      return false;
+    }
+    if (!tesseract::DeSerialize(fp, &empty_)) {
+      return false;
+    }
     if (swap) {
-      for (int i = 0; i < size; ++i)
+      ReverseN(&empty_, sizeof(empty_));
+    }
+    int size = num_elements();
+    if (!tesseract::DeSerialize(fp, &array_[0], size)) {
+      return false;
+    }
+    if (swap) {
+      for (int i = 0; i < size; ++i) {
         ReverseN(&array_[i], sizeof(array_[i]));
+      }
     }
     return true;
   }
 
-  bool DeSerialize(TFile* fp) {
-    return DeSerializeSize(fp) &&
-           fp->DeSerialize(&empty_) &&
+  bool DeSerialize(TFile *fp) {
+    return DeSerializeSize(fp) && fp->DeSerialize(&empty_) &&
            fp->DeSerialize(&array_[0], num_elements());
   }
 
   // Writes to the given file. Returns false in case of error.
   // Assumes a T::Serialize(FILE*) const function.
-  bool SerializeClasses(FILE* fp) const {
-    if (!SerializeSize(fp)) return false;
-    if (!empty_.Serialize(fp)) return false;
+  bool SerializeClasses(FILE *fp) const {
+    if (!SerializeSize(fp)) {
+      return false;
+    }
+    if (!empty_.Serialize(fp)) {
+      return false;
+    }
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
-      if (!array_[i].Serialize(fp)) return false;
+      if (!array_[i].Serialize(fp)) {
+        return false;
+      }
     }
     return true;
   }
@@ -196,22 +220,34 @@ class GENERIC_2D_ARRAY {
   // Reads from the given file. Returns false in case of error.
   // Assumes a T::DeSerialize(bool swap, FILE*) function.
   // If swap is true, assumes a big/little-endian swap is needed.
-  bool DeSerializeClasses(bool swap, FILE* fp) {
-    if (!DeSerializeSize(swap, fp)) return false;
-    if (!empty_.DeSerialize(swap, fp)) return false;
+  bool DeSerializeClasses(bool swap, FILE *fp) {
+    if (!DeSerializeSize(swap, fp)) {
+      return false;
+    }
+    if (!empty_.DeSerialize(swap, fp)) {
+      return false;
+    }
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
-      if (!array_[i].DeSerialize(swap, fp)) return false;
+      if (!array_[i].DeSerialize(swap, fp)) {
+        return false;
+      }
     }
     return true;
   }
 
   // Provide the dimensions of this rectangular matrix.
-  int dim1() const { return dim1_; }
-  int dim2() const { return dim2_; }
+  int dim1() const {
+    return dim1_;
+  }
+  int dim2() const {
+    return dim2_;
+  }
   // Returns the number of elements in the array.
   // Banded/triangular matrices may override.
-  virtual int num_elements() const { return dim1_ * dim2_; }
+  virtual int num_elements() const {
+    return dim1_ * dim2_;
+  }
 
   // Expression to select a specific location in the matrix. The matrix is
   // stored COLUMN-major, so the left-most index is the most significant.
@@ -221,10 +257,10 @@ class GENERIC_2D_ARRAY {
   }
 
   // Put a list element into the matrix at a specific location.
-  void put(ICOORD pos, const T& thing) {
+  void put(ICOORD pos, const T &thing) {
     array_[this->index(pos.x(), pos.y())] = thing;
   }
-  void put(int column, int row, const T& thing) {
+  void put(int column, int row, const T &thing) {
     array_[this->index(column, row)] = thing;
   }
 
@@ -236,23 +272,23 @@ class GENERIC_2D_ARRAY {
     return array_[this->index(column, row)];
   }
   // Return a reference to the element at the specified location.
-  const T& operator()(int column, int row) const {
+  const T &operator()(int column, int row) const {
     return array_[this->index(column, row)];
   }
-  T& operator()(int column, int row) {
+  T &operator()(int column, int row) {
     return array_[this->index(column, row)];
   }
   // Allow access using array[column][row]. NOTE that the indices are
   // in the same left-to-right order as the () indexing.
-  T* operator[](int column) {
+  T *operator[](int column) {
     return &array_[this->index(column, 0)];
   }
-  const T* operator[](int column) const {
+  const T *operator[](int column) const {
     return &array_[this->index(column, 0)];
   }
 
   // Adds addend to *this, element-by-element.
-  void operator+=(const GENERIC_2D_ARRAY<T>& addend) {
+  void operator+=(const GENERIC_2D_ARRAY<T> &addend) {
     if (dim2_ == addend.dim2_) {
       // Faster if equal size in the major dimension.
       int size = std::min(num_elements(), addend.num_elements());
@@ -268,7 +304,7 @@ class GENERIC_2D_ARRAY {
     }
   }
   // Subtracts minuend from *this, element-by-element.
-  void operator-=(const GENERIC_2D_ARRAY<T>& minuend) {
+  void operator-=(const GENERIC_2D_ARRAY<T> &minuend) {
     if (dim2_ == minuend.dim2_) {
       // Faster if equal size in the major dimension.
       int size = std::min(num_elements(), minuend.num_elements());
@@ -284,21 +320,21 @@ class GENERIC_2D_ARRAY {
     }
   }
   // Adds addend to all elements.
-  void operator+=(const T& addend) {
+  void operator+=(const T &addend) {
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
       array_[i] += addend;
     }
   }
   // Multiplies *this by factor, element-by-element.
-  void operator*=(const T& factor) {
+  void operator*=(const T &factor) {
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
       array_[i] *= factor;
     }
   }
   // Clips *this to the given range.
-  void Clip(const T& rangemin, const T& rangemax) {
+  void Clip(const T &rangemin, const T &rangemax) {
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
       array_[i] = ClipToRange(array_[i], rangemin, rangemax);
@@ -306,19 +342,22 @@ class GENERIC_2D_ARRAY {
   }
   // Returns true if all elements of *this are within the given range.
   // Only uses operator<
-  bool WithinBounds(const T& rangemin, const T& rangemax) const {
+  bool WithinBounds(const T &rangemin, const T &rangemax) const {
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
-      const T& value = array_[i];
-      if (value < rangemin || rangemax < value)
+      const T &value = array_[i];
+      if (value < rangemin || rangemax < value) {
         return false;
+      }
     }
     return true;
   }
   // Normalize the whole array.
   double Normalize() {
     int size = num_elements();
-    if (size <= 0) return 0.0;
+    if (size <= 0) {
+      return 0.0;
+    }
     // Compute the mean.
     double mean = 0.0;
     for (int i = 0; i < size; ++i) {
@@ -345,12 +384,16 @@ class GENERIC_2D_ARRAY {
   // Returns the maximum value of the array.
   T Max() const {
     int size = num_elements();
-    if (size <= 0) return empty_;
+    if (size <= 0) {
+      return empty_;
+    }
     // Compute the max.
     T max_value = array_[0];
     for (int i = 1; i < size; ++i) {
-      const T& value = array_[i];
-      if (value > max_value) max_value = value;
+      const T &value = array_[i];
+      if (value > max_value) {
+        max_value = value;
+      }
     }
     return max_value;
   }
@@ -358,30 +401,33 @@ class GENERIC_2D_ARRAY {
   // Returns the maximum absolute value of the array.
   T MaxAbs() const {
     int size = num_elements();
-    if (size <= 0) return empty_;
+    if (size <= 0) {
+      return empty_;
+    }
     // Compute the max.
     T max_abs = static_cast<T>(0);
     for (int i = 0; i < size; ++i) {
       T value = static_cast<T>(fabs(array_[i]));
-      if (value > max_abs) max_abs = value;
+      if (value > max_abs) {
+        max_abs = value;
+      }
     }
     return max_abs;
   }
 
   // Accumulates the element-wise sums of squares of src into *this.
-  void SumSquares(const GENERIC_2D_ARRAY<T>& src, const T& decay_factor) {
-    T update_factor = 1.0 - decay_factor;
+  void SumSquares(const GENERIC_2D_ARRAY<T> &src, const T &decay_factor) {
+    T update_factor = 1 - decay_factor;
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
-      array_[i] = array_[i] * decay_factor +
-                  update_factor * src.array_[i] * src.array_[i];
+      array_[i] = array_[i] * decay_factor + update_factor * src.array_[i] * src.array_[i];
     }
   }
 
   // Scales each element using the adam algorithm, ie array_[i] by
   // sqrt(sqsum[i] + epsilon)).
-  void AdamUpdate(const GENERIC_2D_ARRAY<T>& sum,
-                  const GENERIC_2D_ARRAY<T>& sqsum, const T& epsilon) {
+  void AdamUpdate(const GENERIC_2D_ARRAY<T> &sum, const GENERIC_2D_ARRAY<T> &sqsum,
+                  const T &epsilon) {
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
       array_[i] += sum.array_[i] / (sqrt(sqsum.array_[i]) + epsilon);
@@ -419,8 +465,8 @@ class GENERIC_2D_ARRAY {
   // NOTE: the 2 stored matrix dimensions are simply copied from *this. To
   // change the dimensions after the transpose, use ResizeNoInit.
   // Higher dimensions above 2 are strictly the responsibility of the caller.
-  void RotatingTranspose(const int* dims, int num_dims, int src_dim,
-                         int dest_dim, GENERIC_2D_ARRAY<T>* result) const {
+  void RotatingTranspose(const int *dims, int num_dims, int src_dim, int dest_dim,
+                         GENERIC_2D_ARRAY<T> *result) const {
     int max_d = std::max(src_dim, dest_dim);
     int min_d = std::min(src_dim, dest_dim);
     // In a tensor of shape [d0, d1... min_d, ... max_d, ... dn-2, dn-1], the
@@ -432,18 +478,28 @@ class GENERIC_2D_ARRAY {
     // src_step represents the stride in the src between each adjacent group
     // in the destination.
     int num_replicas = 1, move_size = 1, src_step = 1;
-    for (int d = 0; d < min_d; ++d) num_replicas *= dims[d];
-    for (int d = max_d + 1; d < num_dims; ++d) move_size *= dims[d];
-    for (int d = src_dim + 1; d < num_dims; ++d) src_step *= dims[d];
-    if (src_dim > dest_dim) src_step *= dims[src_dim];
+    for (int d = 0; d < min_d; ++d) {
+      num_replicas *= dims[d];
+    }
+    for (int d = max_d + 1; d < num_dims; ++d) {
+      move_size *= dims[d];
+    }
+    for (int d = src_dim + 1; d < num_dims; ++d) {
+      src_step *= dims[d];
+    }
+    if (src_dim > dest_dim) {
+      src_step *= dims[src_dim];
+    }
     // wrap_size is the size of a single replica, being the amount that is
     // handled num_replicas times.
     int wrap_size = move_size;
-    for (int d = min_d; d <= max_d; ++d) wrap_size *= dims[d];
+    for (int d = min_d; d <= max_d; ++d) {
+      wrap_size *= dims[d];
+    }
     result->ResizeNoInit(dim1_, dim2_);
     result->empty_ = empty_;
-    const T* src = array_;
-    T* dest = result->array_;
+    const T *src = array_;
+    T *dest = result->array_;
     for (int replica = 0; replica < num_replicas; ++replica) {
       for (int start = 0; start < src_step; start += move_size) {
         for (int pos = start; pos < wrap_size; pos += src_step) {
@@ -460,56 +516,77 @@ class GENERIC_2D_ARRAY {
     int size = num_elements();
     for (int i = 0; i < size; ++i) {
       T matrix_cell = array_[i];
-      if (matrix_cell != empty_)
+      if (matrix_cell != empty_) {
         delete matrix_cell;
+      }
     }
   }
 
- protected:
+protected:
   // Factored helper to serialize the size.
-  bool SerializeSize(FILE* fp) const {
+  bool SerializeSize(FILE *fp) const {
     uint32_t size = dim1_;
-    if (!tesseract::Serialize(fp, &size)) return false;
+    if (!tesseract::Serialize(fp, &size)) {
+      return false;
+    }
     size = dim2_;
     return tesseract::Serialize(fp, &size);
   }
-  bool SerializeSize(TFile* fp) const {
+  bool SerializeSize(TFile *fp) const {
     uint32_t size = dim1_;
-    if (!fp->Serialize(&size)) return false;
+    if (!fp->Serialize(&size)) {
+      return false;
+    }
     size = dim2_;
     return fp->Serialize(&size);
   }
   // Factored helper to deserialize the size.
   // If swap is true, assumes a big/little-endian swap is needed.
-  bool DeSerializeSize(bool swap, FILE* fp) {
+  bool DeSerializeSize(bool swap, FILE *fp) {
     uint32_t size1, size2;
-    if (!tesseract::DeSerialize(fp, &size1)) return false;
-    if (!tesseract::DeSerialize(fp, &size2)) return false;
+    if (!tesseract::DeSerialize(fp, &size1)) {
+      return false;
+    }
+    if (!tesseract::DeSerialize(fp, &size2)) {
+      return false;
+    }
     if (swap) {
       ReverseN(&size1, sizeof(size1));
       ReverseN(&size2, sizeof(size2));
     }
     // Arbitrarily limit the number of elements to protect against bad data.
-    if (size1 > UINT16_MAX) return false;
-    if (size2 > UINT16_MAX) return false;
+    if (size1 > UINT16_MAX) {
+      return false;
+    }
+    if (size2 > UINT16_MAX) {
+      return false;
+    }
     Resize(size1, size2, empty_);
     return true;
   }
-  bool DeSerializeSize(TFile* fp) {
+  bool DeSerializeSize(TFile *fp) {
     int32_t size1, size2;
-    if (!fp->DeSerialize(&size1)) return false;
-    if (!fp->DeSerialize(&size2)) return false;
+    if (!fp->DeSerialize(&size1)) {
+      return false;
+    }
+    if (!fp->DeSerialize(&size2)) {
+      return false;
+    }
     // Arbitrarily limit the number of elements to protect against bad data.
-    if (size1 > UINT16_MAX) return false;
-    if (size2 > UINT16_MAX) return false;
+    if (size1 > UINT16_MAX) {
+      return false;
+    }
+    if (size2 > UINT16_MAX) {
+      return false;
+    }
     Resize(size1, size2, empty_);
     return true;
   }
 
-  T* array_;
-  T empty_;   // The unused cell.
-  int dim1_;  // Size of the 1st dimension in indexing functions.
-  int dim2_;  // Size of the 2nd dimension in indexing functions.
+  T *array_;
+  T empty_;  // The unused cell.
+  int dim1_; // Size of the 1st dimension in indexing functions.
+  int dim2_; // Size of the 2nd dimension in indexing functions.
   // The total size to which the array can be expanded before a realloc is
   // needed. If Resize is used, memory is retained so it can be re-expanded
   // without a further alloc, and this stores the allocated size.
@@ -523,20 +600,22 @@ class GENERIC_2D_ARRAY {
 // assert will fail if row < col or row - col >= dim2.
 template <class T>
 class BandTriMatrix : public GENERIC_2D_ARRAY<T> {
- public:
+public:
   // Allocate a piece of memory to hold a 2d-array of the given dimension.
   // Initialize all the elements of the array to empty instead of assuming
   // that a default constructor can be used.
-  BandTriMatrix(int dim1, int dim2, const T& empty)
-    : GENERIC_2D_ARRAY<T>(dim1, dim2, empty)  {
-  }
+  BandTriMatrix(int dim1, int dim2, const T &empty) : GENERIC_2D_ARRAY<T>(dim1, dim2, empty) {}
   // The default destructor will do.
 
   // Provide the dimensions of this matrix.
   // dimension is the size of the nominally square matrix.
-  int dimension() const { return this->dim1_; }
+  int dimension() const {
+    return this->dim1_;
+  }
   // bandwidth is the number of bands in the matrix, INCLUDING the diagonal.
-  int bandwidth() const { return this->dim2_; }
+  int bandwidth() const {
+    return this->dim2_;
+  }
 
   // Expression to select a specific location in the matrix. The matrix is
   // stored COLUMN-major, so the left-most index is the most significant.
@@ -551,18 +630,17 @@ class BandTriMatrix : public GENERIC_2D_ARRAY<T> {
   // equal to the sum of the individual dimensions.
   // array2 is not destroyed, but is left empty, as all elements are moved
   // to *this.
-  void AttachOnCorner(BandTriMatrix<T>* array2) {
+  void AttachOnCorner(BandTriMatrix<T> *array2) {
     int new_dim1 = this->dim1_ + array2->dim1_;
     int new_dim2 = std::max(this->dim2_, array2->dim2_);
-    T* new_array = new T[new_dim1 * new_dim2];
+    T *new_array = new T[new_dim1 * new_dim2];
     for (int col = 0; col < new_dim1; ++col) {
       for (int j = 0; j < new_dim2; ++j) {
         int new_index = col * new_dim2 + j;
         if (col < this->dim1_ && j < this->dim2_) {
           new_array[new_index] = this->get(col, col + j);
         } else if (col >= this->dim1_ && j < array2->dim2_) {
-          new_array[new_index] = array2->get(col - this->dim1_,
-                                             col - this->dim1_ + j);
+          new_array[new_index] = array2->get(col - this->dim1_, col - this->dim1_ + j);
           array2->put(col - this->dim1_, col - this->dim1_ + j, nullptr);
         } else {
           new_array[new_index] = this->empty_;
@@ -577,9 +655,9 @@ class BandTriMatrix : public GENERIC_2D_ARRAY<T> {
 };
 
 class MATRIX : public BandTriMatrix<BLOB_CHOICE_LIST *> {
- public:
+public:
   MATRIX(int dimension, int bandwidth)
-    : BandTriMatrix<BLOB_CHOICE_LIST *>(dimension, bandwidth, NOT_CLASSIFIED) {}
+      : BandTriMatrix<BLOB_CHOICE_LIST *>(dimension, bandwidth, NOT_CLASSIFIED) {}
 
   ~MATRIX() override;
 
@@ -595,12 +673,12 @@ class MATRIX : public BandTriMatrix<BLOB_CHOICE_LIST *> {
   // Entries are relocated to the new MATRIX using the transformation defined
   // by MATRIX_COORD::MapForSplit.
   // Transfers the pointer data to the new MATRIX and deletes *this.
-  MATRIX* ConsumeAndMakeBigger(int ind);
+  MATRIX *ConsumeAndMakeBigger(int ind);
 
   // Makes and returns a deep copy of *this, including all the BLOB_CHOICEs
   // on the lists, but not any LanguageModelState that may be attached to the
   // BLOB_CHOICEs.
-  MATRIX* DeepCopy() const;
+  MATRIX *DeepCopy() const;
 
   // Print a shortened version of the contents of the matrix.
   void print(const UNICHARSET &unicharset) const;
@@ -613,12 +691,12 @@ struct MATRIX_COORD {
   }
   // Default constructor required by GenericHeap.
   MATRIX_COORD() : col(0), row(0) {}
-  MATRIX_COORD(int c, int r): col(c), row(r) {}
-  ~MATRIX_COORD() {}
+  MATRIX_COORD(int c, int r) : col(c), row(r) {}
+  ~MATRIX_COORD() = default;
 
   bool Valid(const MATRIX &m) const {
-    return 0 <= col && col < m.dimension() &&
-           col <= row && row < col + m.bandwidth() && row < m.dimension();
+    return 0 <= col && col < m.dimension() && col <= row && row < col + m.bandwidth() &&
+           row < m.dimension();
   }
 
   // Remaps the col,row pair to split the blob at the given (ind,ind) diagonal
@@ -629,8 +707,12 @@ struct MATRIX_COORD {
   // making a new column at ind+1.
   void MapForSplit(int ind) {
     ASSERT_HOST(row >= col);
-    if (col > ind) ++col;
-    if (row >= ind) ++row;
+    if (col > ind) {
+      ++col;
+    }
+    if (row >= ind) {
+      ++row;
+    }
     ASSERT_HOST(row >= col);
   }
 
@@ -641,6 +723,6 @@ struct MATRIX_COORD {
 // The MatrixCoordPair contains a MATRIX_COORD and its priority.
 using MatrixCoordPair = KDPairInc<float, MATRIX_COORD>;
 
-}  // namespace tesseract
+} // namespace tesseract
 
-#endif  // TESSERACT_CCSTRUCT_MATRIX_H_
+#endif // TESSERACT_CCSTRUCT_MATRIX_H_
